# SPDX-License-Identifier: BSD-2-Clause

from .._util import _deprecated_import

__all__ = (
	'quickfeather',
)

def __dir__() -> list[str]:
	return list({*globals(), *__all__})

def __getattr__(name: str):
	match name:
		case 'quickfeather':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.eos_s3.{name}')
			from .eos_s3 import quickfeather
			return quickfeather
		case _:
			raise AttributeError(f'module {__name__!r} has no attribute {name!r}')



# SPDX-License-Identifier: BSD-2-Clause

from textwrap                           import dedent
from typing                             import Literal

from torii.build                        import Attrs, Clock, Connector, DiffPairs, Pins, PinsN, Resource, Subsignal
from torii.build.run                    import BuildPlan, BuildProducts
from torii.hdl.ir                       import Fragment
from torii.platform.resources.display   import VGAResource
from torii.platform.resources.interface import I2CResource, SPIResource, UARTResource, ULPIResource
from torii.platform.resources.memory    import SDCardResources
from torii.platform.resources.user      import ButtonResources, LEDResources, SwitchResources
from torii.platform.vendor.xilinx       import XilinxPlatform

__all__ = (
	'Genesys2Platform',
)

class Genesys2Platform(XilinxPlatform):
	'''Platform file for Diglient Genesys2 Kitex-7 board.
	https://digilent.com/reference/programmable-logic/genesys-2/start'''

	device  = 'xc7k325t'
	package = 'ffg900'
	speed   = '2'

	pretty_name = 'Gensys 2'
	description = 'Digilent Gensys 2 Xilinx Kintex7-325T Development Board'

	def __init__(self, JP6: Literal['1V2', '1V8', '2V5', '3V3'] = '2V5') -> None:
		super().__init__()

		if JP6 not in ('1V2', '1V8', '2V5', '3V3'):
			raise ValueError(f'JP6 must be one of \'1V2\', \'1V8\', \'2V5\', \'3V3\', not {JP6!r}')

		self._JP6 = JP6

	def bank15_16_17_iostandard(self) -> str:
		return 'LVCMOS' + self._JP6.replace('V', '')

	default_rst = 'rst'
	default_clk = 'clk'

	resources = [
		Resource('rst', 0, PinsN('R19', dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),
		Resource(
			'clk', 0, DiffPairs(p = 'AD12 ', n = 'AD11', dir = 'i'), Clock(200e6),
			Attrs(IOSTANDARD = 'LVDS')
		),
		*ButtonResources(pins = {
				0: 'E18',
				1: 'B19',
				2: 'C19',
				3: 'M19',
				4: 'M20'
			}, attrs = Attrs(IOSTANDARD = bank15_16_17_iostandard)
		),
		*SwitchResources(
			pins = 'G19 G25 H24 K19 N19 P19',
			attrs = Attrs(IOSTANDARD = bank15_16_17_iostandard)
		),
		*SwitchResources(pins = {
				6: 'P26',
				7: 'P27'
			}, attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		*LEDResources(
			pins = 'T28 V19 U30 U29 V20 V26 W24 W23',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource(
			'fan', 0,
			Subsignal('pwm', Pins('W19', dir = 'o')),
			Subsignal('tach', Pins('V21', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
		UARTResource(
			0,
			rx = 'Y20', tx = 'Y23',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		I2CResource(
			0,
			scl = 'AE30', sda = 'AF30',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		# TODO(aki): Replace with `DDR3Resource` when more granular attrs are possible
		Resource(
			'ddr3', 0,
			Subsignal('rst', PinsN('AG5', dir = 'o'), Attrs(IOSTANDARD = 'SSTL15')),
			Subsignal(
				'clk',
				DiffPairs(p = 'AB9', n = 'AC9', dir = 'o'), Attrs(IOSTANDARD = 'DIFF_SSTL15_DCI')
			),
			Subsignal('clk_en', Pins('AJ9', dir = 'o')),
			Subsignal('cs', PinsN('AH12', dir = 'o')),
			Subsignal('we', PinsN('AG13', dir = 'o')),
			Subsignal('ras', PinsN('AE11', dir = 'o')),
			Subsignal('cas', PinsN('AF11', dir = 'o')),
			Subsignal('a', Pins(
				'AC12 AE8 AD8 AC10 AD9 AA13 AA10 AA11 '
				'Y10 Y11 AB8 AA8 AB12 AA12 AH9 AG9', dir = 'o')
			),
			Subsignal('ba', Pins('AE9 AB10 AC11', dir = 'o')),
			Subsignal(
				'dqs',
				DiffPairs(p = 'AD2 AG4 AG2 AH7', n = 'AD1 AG3 AH1 AJ7', dir = 'io'),
				Attrs(IOSTANDARD = 'DIFF_SSTL15_DCI', ODT = 'RTT_40')
			),
			Subsignal(
				'dq',
				Pins(
					'AD3 AC2 AC1 AC5 AC4 AD6 AE6 AC7 '
					'AF2 AE1 AF1 AE4 AE3 AE5 AF5 AF6 '
					'AJ4 AH6 AH5 AH2 AJ2 AJ1 AK1 AJ3 '
					'AF7 AG7 AJ6 AK6 AJ8 AK8 AK5 AK4',
					dir = 'io'
				),
				Attrs(ODT = 'RTT_40')
			),
			Subsignal('dm', Pins('AD4 AF3 AH4 AF8', dir = 'o')),
			Subsignal('odt', Pins('AK9', dir = 'o')),
			Attrs(IOSTANDARD = 'SSTL15_DCI', SLEW = 'FAST', OUTPUT_IMPEDANCE = 'RDRV_40_40')
		),
		Resource(
			'audio_i2c', 0,  # ADAU1761 I2C
			Subsignal('scl', Pins('AE19', dir = 'io')),
			Subsignal('sda', Pins('AF18', dir = 'io')),
			Subsignal('adr', Pins('AD19 AG19', dir = 'o')),
			Attrs(IOSTANDARD = 'LVCMOS18')
		),
		Resource(
			'audio_i2s', 0,  # ADAU1761 ADC, I2S
			Subsignal('clk', Pins('AG18', dir = 'o')), # BCLK
			Subsignal('sd_adc', Pins('AH19', dir = 'o')), # ADC_SDATA
			Subsignal('sd_dac', Pins('AJ19', dir = 'i')),  # DAC_SDATA
			Subsignal('ws', Pins('AJ18', dir = 'o')), # LRCLK
			Attrs(IOSTANDARD = 'LVCMOS18')
		),
		Resource(
			'audio_clk', 0,  # ADAU1761 MCLK
			Pins('AK19', dir = 'o'), Attrs(IOSTANDARD = 'LVCMOS18')
		),
		SPIResource(
			0,  # OLED, SSD1306, 128 x 32
			cs_n = 'dummy-cs0', clk = 'AF17', copi = 'Y15',
			cipo = 'dummy-cipo0', reset = 'AB17',
			attrs = Attrs(IOSTANDARD = 'LVCMOS18')
		),
		Resource(
			'oled', 0,  # OLED, UG-2832HSWEG04
			Subsignal('dc', Pins('AC17', dir = 'o')),
			Subsignal('vdd_en', PinsN('AG17', dir = 'o')),
			Subsignal('vbat_en', PinsN('AB22', dir = 'o'), Attrs(IOSTANDARD = 'LVCMOS33')),
			Attrs(IOSTANDARD = 'LVCMOS18')
		),
		# TODO(aki): Replace with `HDMIResource` when merged + released in Torii
		Resource(
			'hdmi', 0,  # HDMI TX, connector J4
			Subsignal('scl', Pins('AF27', dir = 'io'), Attrs(IOSTANDARD = 'LVCMOS33')),
			Subsignal('sda', Pins('AF26', dir = 'io'), Attrs(IOSTANDARD = 'LVCMOS33')),
			Subsignal('clk', DiffPairs(p = 'AA20', n = 'AB20', dir = 'o')),
			Subsignal('d', DiffPairs(
				p = 'AC20 AA22 AB24',
				n = 'AC21 AA23 AC25',
				dir = 'o'
			)),
			Attrs(IOSTANDARD = 'TMDS_33')
		),
		# TODO(aki): Replace with `HDMIResource` when merged + released in Torii
		Resource(
			'hdmi', 1,  # HDMI RX, connector J5
			Subsignal('scl', Pins('AJ28', dir = 'io'), Attrs(IOSTANDARD = 'LVCMOS33')),
			Subsignal('sda', Pins('AJ29', dir = 'io'), Attrs(IOSTANDARD = 'LVCMOS33')),
			Subsignal('clk', DiffPairs(p = 'AE28', n = 'AF28', dir = 'i')),
			Subsignal('rx', DiffPairs(
				p = 'AJ26 AG27 AH26',
				n = 'AK26 AG28 AH27',
				dir = 'i'
			)),
			Attrs(IOSTANDARD = 'TMDS_33')
		),
		VGAResource(
			0,
			r = 'AK25 AG25 AH25 AK24 AJ24',
			g = 'AJ23 AJ22 AH22 AK21 AJ21 AK23',
			b = 'AH20 AG20 AF21 AK20 AG22',
			hs = 'AF20', vs = 'AG23', invert_sync = True,
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		*SDCardResources(
			0,
			clk = 'R28', cmd = 'R29', dat0 = 'R26', dat1 = 'R30',
			dat2 = 'P29', dat3 = 'T30', cd = 'P28',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource(
			'sd_card_rst', 0,
			Pins('AE24', dir = 'o'), Attrs(IOSTANDARD = 'LVCMOS33')
		),
		ULPIResource(
			0,
			data = 'AE14 AE15 AC15 AC16 AB15 AA15 AD14 AC14',
			rst = 'AB14', clk = 'AD18', dir = 'Y16', stp = 'AA17', nxt = 'AA16',
			clk_dir = 'i', rst_invert = True, attrs = Attrs(IOSTANDARD = 'LVCMOS18')
		),
		Resource(
			'vusb_oc', 0, PinsN('AF16', dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS18')
		),
		# TODO(aki): Replace with `EthernetResource` when more granular attrs are possible
		Resource(
			'eth_rgmii', 0,
			Subsignal('rst', PinsN('AH24', dir = 'o'), Attrs(IOSTANDARD = 'LVCMOS33')),
			Subsignal('mdc', Pins('AF12', dir = 'o')),
			Subsignal('mdio', Pins('AG12', dir = 'io')),
			Subsignal('tx_clk', Pins('AE10', dir = 'o')),
			Subsignal('tx_ctl', Pins('AK14', dir = 'o')),
			Subsignal('tx_data', Pins('AJ12 AK11 AJ11 AK10', dir = 'o')),
			Subsignal('rx_clk', Pins('AG10', dir = 'i')),
			Subsignal('rx_ctl', Pins('AH11', dir = 'i')),
			Subsignal('rx_data', Pins('AJ14 AH14 AK13 AJ13', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS15')
		)
	]

	connectors = [
		Connector(
			'pmod', 0,  # JA
			'U27 U28 T26 T27 - - '
			'T22 T23 T20 T21 - - '
		),
		Connector(
			'pmod', 1,  # JB
			'V29 V30 V25 W26 - - '
			'T25 U25 U22 U23 - - '
		),
		Connector(
			'pmod', 2,  # JC
			'AC26 AJ27 AH30 AK29 - - '
			'AD26 AG30 AK30 AK28 - - '
		),
		Connector(
			'pmod', 3,  # JD
			'V27 Y30 V24 W22 - - '
			'U24 Y26 V22 W21 - - '
		),
		Connector(
			'pmod', 4,  # JXADC
			'J23 K23 L22 L21 - - '
			'J24 K24 L23 K21 - - '
		),
		Connector('hpc', 0, {
			'dp1_m2c_p': 'Y6',
			'dp1_m2c_n': 'Y5',
			'dp2_m2c_p': 'W4',
			'dp2_m2c_n': 'W3',
			'dp3_m2c_p': 'V6',
			'dp3_m2c_n': 'V5',
			'dp1_c2m_p': 'V2',
			'dp1_c2m_n': 'V1',
			'dp2_c2m_p': 'U4',
			'dp2_c2m_n': 'U3',
			'dp3_c2m_p': 'T2',
			'dp3_c2m_n': 'T1',
			'dp0_c2m_p': 'Y2',
			'dp0_c2m_n': 'Y1',
			'dp0_m2c_p': 'AA4',
			'dp0_m2c_n': 'AA3',
			'la06_p': 'D29',
			'la06_n': 'C30',
			'la10_p': 'B27',
			'la10_n': 'A27',
			'la14_p': 'C24',
			'la14_n': 'B24',
			'la18_cc_p': 'D17',
			'la18_cc_n': 'D18',
			'la27_p': 'A20',
			'la27_n': 'A21',
			'ha01_cc_p': 'M28',
			'ha01_cc_n': 'L28',
			'ha05_p': 'J29',
			'ha05_n': 'H29',
			'ha09_p': 'L30',
			'ha09_n': 'K30',
			'ha13_p': 'K26',
			'ha13_n': 'J26',
			'ha16_p': 'M22',
			'ha16_n': 'M23',
			'ha20_p': 'G27',
			'ha20_n': 'F27',
			'clk1_m2c_p': 'E28',
			'clk1_m2c_n': 'D28',
			'la00_cc_p': 'D27',
			'la00_cc_n': 'C27',
			'la03_p': 'E29',
			'la03_n': 'E30',
			'la08_p': 'C29',
			'la08_n': 'B29',
			'la12_p': 'F26',
			'la12_n': 'E26',
			'la16_p': 'E23',
			'la16_n': 'D23',
			'la20_p': 'G22',
			'la20_n': 'F22',
			'la22_p': 'J17',
			'la22_n': 'H17',
			'la25_p': 'D22',
			'la25_n': 'C22',
			'la29_p': 'B18',
			'la29_n': 'A18',
			'la31_p': 'C17',
			'la31_n': 'B17',
			'la33_p': 'D16',
			'la33_n': 'C16',
			'ha03_p': 'N25',
			'ha03_n': 'N26',
			'ha07_p': 'M29',
			'ha07_n': 'M30',
			'ha11_p': 'P23',
			'ha11_n': 'N24',
			'ha14_p': 'N27',
			'ha14_n': 'M27',
			'ha18_p': 'E19',
			'ha18_n': 'D19',
			'ha22_p': 'D21',
			'ha22_n': 'C21',
			'gbtclk1_m2c_p': 'N8',
			'gbtclk1_m2c_n': 'N7',
			'gbtclk0_m2c_p': 'L8',
			'gbtclk0_m2c_n': 'L7',
			'la01_cc_p': 'D26',
			'la01_cc_n': 'C26',
			'la05_p': 'B30',
			'la05_n': 'A30',
			'la09_p': 'B28',
			'la09_n': 'A28',
			'la13_p': 'E24',
			'la13_n': 'D24',
			'la17_cc_p': 'F21',
			'la17_cc_n': 'E21',
			'la23_p': 'G17',
			'la23_n': 'F17',
			'la26_p': 'B22',
			'la26_n': 'A22',
			'pg_m2c': 'AH21',
			'ha00_cc_p': 'K28',
			'ha00_cc_n': 'K29',
			'ha04_p': 'M24',
			'ha04_n': 'M25',
			'ha08_p': 'J27',
			'ha08_n': 'J28',
			'ha12_p': 'L26',
			'ha12_n': 'L27',
			'ha15_p': 'J21',
			'ha15_n': 'J22',
			'ha19_p': 'G29',
			'ha19_n': 'F30',
			'prsnt_m2c_b': 'AA21',
			'clk0_m2c_p': 'F20',
			'clk0_m2c_n': 'E20',
			'la02_p': 'H30',
			'la02_n': 'G30',
			'la04_p': 'H26',
			'la04_n': 'H27',
			'la07_p': 'F25',
			'la07_n': 'E25',
			'la11_p': 'A25',
			'la11_n': 'A26',
			'la15_p': 'B23',
			'la15_n': 'A23',
			'la19_p': 'H21',
			'la19_n': 'H22',
			'la21_p': 'L17',
			'la21_n': 'L18',
			'la24_p': 'H20',
			'la24_n': 'G20',
			'la28_p': 'J19',
			'la28_n': 'H19',
			'la30_p': 'A16',
			'la30_n': 'A17',
			'la32_p': 'K18',
			'la32_n': 'J18',
			'ha02_p': 'P21',
			'ha02_n': 'P22',
			'ha06_p': 'N29',
			'ha06_n': 'N30',
			'ha10_p': 'N21',
			'ha10_n': 'N22',
			'ha17_cc_p': 'C25',
			'ha17_cc_n': 'B25',
			'ha21_p': 'G28',
			'ha21_n': 'F28',
			'ha23_p': 'G18',
			'ha23_n': 'F18'
		})
	]

	def toolchain_prepare(self, fragment: Fragment, name: str, **kwargs) -> BuildPlan:
		overrides = {
			'script_after_read': 'auto_detect_xpm',
			'script_before_bitstream':
			'set_property BITSTREAM.GENERAL.COMPRESS TRUE [current_design]',
			'add_constraints': dedent('''
			set_property BITSTREAM.CONFIG.CONFIGRATE 50 [current_design]
			set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]
			set_property BITSTREAM.CONFIG.SPI_32BIT_ADDR YES [current_design]
			set_property BITSTREAM.CONFIG.SPI_FALL_EDGE YES [current_design]
			set_property CFGBVS VCCO [current_design]
			set_property CONFIG_VOLTAGE 3.3 [current_design]
			''')
		}
		return super().toolchain_prepare(
			fragment, name, **overrides, **kwargs
		)

	@property
	def file_templates(self) -> dict[str, str]:
		return {
			**super().file_templates,
			'{{name}}-openocd.cfg': dedent(r'''
			source [find interface/ftdi/digilent-hs1.cfg]
			# fix channel
			ftdi_channel 1
			adapter_khz 25000
			transport select jtag
			source [find cpld/xilinx-xc7.cfg]
			source [find cpld/jtagspi.cfg]
			''')
		}

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		openocd = environ.get('OPENOCD', 'openocd')
		with products.extract(f'{name}-openocd.cfg', f'{name}.bit') as files:
			(config_filename, bitstream_filename) = files

			check_call([
				openocd,
				'-f', config_filename,
				'-c', f'init; pld load 0 {bitstream_filename}; exit'
			])


if __name__ == '__main__':
	from ...test.blinky import Blinky
	Genesys2Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                         import Clock, Connector, Pins, Resource
from torii.build.run                     import BuildProducts
from torii.platform.resources.display    import Display7SegResource, VGAResource
from torii.platform.resources.interface  import UARTResource
from torii.platform.resources.memory     import SPIFlashResources
from torii.platform.resources.user       import ButtonResources, LEDResources
from torii.platform.vendor.lattice.ice40 import ICE40Platform

__all__ = (
	'NandlandGoPlatform',
)

class NandlandGoPlatform(ICE40Platform):
	device      = 'iCE40HX1K'
	package     = 'VQ100'
	default_clk = 'clk25'

	pretty_name = 'Nandland Go'
	description = 'Nandland Go Lattice iCE40-HX1K Development Board'

	resources = [
		Resource(
			'clk25', 0, Pins('15', dir = 'i'), Clock(25e6)
		),
		*LEDResources(pins = '56 57 59 60'),
		*ButtonResources(pins = '53 51 54 52'),
		Display7SegResource(
			0,
			a = '3', b = '4', c = '93', d = '91', e = '90', f = '1', g = '2',
			invert = True
		),
		Display7SegResource(
			1,
			a = '100', b = '99', c = '97', d = '95', e = '94', f = '8', g = '96',
			invert = True
		),
		UARTResource(0, rx = '73', tx = '74'),
		*SPIFlashResources(0, cs_n = '49', clk = '48', copi = '45', cipo = '46'),
		VGAResource(
			0,
			r = '36 37 40',
			g = '29 30 33',
			b = '28 41 42',
			hs = '26', vs = '27'
		),
	]
	connectors = [
		Connector('pmod', 0, '65 64 63 62 - - 78 79 80 81 - -'),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		iceprog = environ.get('ICEPROG', 'iceprog')
		with products.extract(f'{name}.bin') as bitstream_filename:
			check_call([iceprog, bitstream_filename])


if __name__ == '__main__':
	from ...test.blinky import Blinky
	NandlandGoPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause
from datetime     import date
from pathlib      import Path

from torii        import __version__ as torii_version
from torii_boards import __version__ as boards_version

ROOT_DIR = (Path(__file__).parent).parent

project = 'Torii-HDL Boards'
version = boards_version
release = version.split('+')[0]
copyright = f'{date.today().year}, Shrine Maiden Heavy Industries'
language  = 'en'
docver    = version

extensions = [
	'sphinx.ext.autodoc',
	'sphinx.ext.extlinks',
	'sphinx.ext.githubpages',
	'sphinx.ext.intersphinx',
	'sphinx.ext.napoleon',
	'sphinx.ext.todo',
	'myst_parser',
	'sphinx_autodoc_typehints',
	'sphinx_codeautolink',
	'sphinx_copybutton',
	'sphinx_inline_tabs',
	'sphinx_multiversion',
	'sphinxext.opengraph',
]

source_suffix = {
	'.rst': 'restructuredtext',
	'.md': 'markdown',
}

extlinks = {
	'issue': ('https://github.com/shrine-maiden-heavy-industries/torii-boards/issues/%s', 'torii-boards/%s'),
	'pypi':  ('https://pypi.org/project/%s/', '%s'),
}

pygments_style              = 'default'
pygments_dark_style         = 'monokai'
autodoc_member_order        = 'bysource'
autodoc_docstring_signature = False
todo_include_todos          = True

intersphinx_mapping = {
	'python': ('https://docs.python.org/3', None),
	'torii': (f'https://torii.shmdn.link/v{torii_version}', None)
}

napoleon_google_docstring              = False
napoleon_numpy_docstring               = True
napoleon_use_ivar                      = True
napoleon_use_admonition_for_notes      = True
napoleon_use_admonition_for_examples   = True
napoleon_use_admonition_for_references = True
napoleon_custom_sections  = [
	('Attributes', 'params_style'),
	'Platform overrides'
]

myst_heading_anchors = 3

always_use_bars_union = True
typehints_defaults = 'braces-after'
typehints_use_signature = True
typehints_use_signature_return = True

templates_path = [
	'_templates',
]

html_baseurl     = 'https://torii-boards.shmdn.link/'
html_theme       = 'furo'
html_copy_source = False

html_theme_options = {
	'announcement': 'This documentation is a work in progress, and you can help us <a href="https://github.com/shrine-maiden-heavy-industries/torii-boards/blob/main/CONTRIBUTING.md">improve it!</a>', # noqa: E501
	'light_css_variables': {
		'color-brand-primary': '#2672a8',
		'color-brand-content': '#2672a8',
		'color-announcement-background': '#ffab87',
		'color-announcement-text': '#494453',
	},
	'dark_css_variables': {
		'color-brand-primary': '#85C2FE',
		'color-brand-content': '#85C2FE',
		'color-announcement-background': '#ffab87',
		'color-announcement-text': '#494453',
	},
	'source_repository': 'https://github.com/shrine-maiden-heavy-industries/torii-boards/',
	'source_branch': 'main',
	'source_directory': 'docs/',
}

html_static_path = [
	'_static'
]

html_sidebars = {
	"**": [
		"sidebar/brand.html",
		"sidebar/search.html",
		"sidebar/scroll-start.html",
		"sidebar/navigation.html",
		"sidebar/version_selector.html",
		"sidebar/scroll-end.html",
	]
}

html_css_files = [
	'css/styles.css'
]

# TODO(aki): OpenGraph metadata stuff
ogp_site_url = html_baseurl
ogp_social_cards = {}
ogp_image = None
ogp_image_alt = None
ogp_custom_meta_tags = list[str]()
ogp_enable_meta_description = True

linkcheck_retries = 2
linkcheck_workers = 1 # At the cost of speed try to prevent rate-limiting
linkcheck_ignore  = [
	'https://www.intel.com/content/www/us/en/products/details/fpga/development-tools/quartus-prime.html',
	'https://www.xilinx.com/products/design-tools/ise-design-suite.html',
	'https://www.xilinx.com/products/design-tools/vivado.html',
	# Digilent is being a big meanie
	'https://digilent.com/reference/programmable-logic/genesys-2/start',
	'https://digilent.com/reference/programmable-logic/atlys/start',
	# SSL Timeout from GitHub actions
	'https://aur.archlinux.org',
]
linkcheck_anchors_ignore_for_url = [
	r'^https://web\.libera\.chat/',
]

# Sphinx-Multiversion stuff
# TODO(aki): Revert to `^v(?!0)\d+\.\d+\.\d+$` when `v1.0.0` drops, we need to gen the v0.8.0 stable for now
smv_tag_whitelist    = r'^v\d+\.(?![1-7])\d+\.\d+$' # Ignore all `v0.[1-7].x` versions
smv_branch_whitelist = r'^main$'                    # Only look at `main`
smv_remote_whitelist = r'^origin$'
smv_released_pattern = r'^refs/tags/v.+$'           # Only consider tags to be full releases
smv_outputdir_format = '{ref.name}'



# SPDX-License-Identifier: BSD-2-Clause

from .colorlight_5a75b_r7_0 import Colorlight_5A75B_R70Platform
from .ecp5_5g_evn           import ECP55GEVNPlatform
from .ecpix5                import ECPIX545Platform, ECPIX585Platform
from .logicbone             import LogicbonePlatform, Logicbone85FPlatform
from .orangecrab_r0_1       import OrangeCrabR0_1Platform
from .orangecrab_r0_2       import OrangeCrabR0_2_25FPlatform, OrangeCrabR0_2_85FPlatform
from .supercon19badge       import Supercon19BadgePlatform
from .ulx3s                 import ULX3S_12F_Platform, ULX3S_25F_Platform, ULX3S_45F_Platform, ULX3S_85F_Platform
from .versa_ecp5_5g         import VersaECP55GPlatform
from .versa_ecp5            import VersaECP5Platform

__all__ = (
	'Colorlight_5A75B_R70Platform',
	'ECP55GEVNPlatform',
	'ECPIX545Platform',
	'ECPIX585Platform',
	'LogicbonePlatform',
	'Logicbone85FPlatform',
	'OrangeCrabR0_1Platform',
	'OrangeCrabR0_2_25FPlatform',
	'OrangeCrabR0_2_85FPlatform',
	'Supercon19BadgePlatform',
	'ULX3S_12F_Platform',
	'ULX3S_25F_Platform',
	'ULX3S_45F_Platform',
	'ULX3S_85F_Platform',
	'VersaECP55GPlatform',
	'VersaECP5Platform',
)



# SPDX-License-Identifier: BSD-2-Clause

from .chameleon96 import Chameleon96Platform
from .de0_cv      import DE0CVPlatform
from .de1_soc     import DE1SoCPlatform
from .de10_nano   import DE10NanoPlatform
from .mister      import MisterPlatform

__all__ = (
	'Chameleon96Platform',
	'DE0CVPlatform',
	'DE1SoCPlatform',
	'DE10NanoPlatform',
	'MisterPlatform',
)



# SPDX-License-Identifier: BSD-2-Clause

from .arty_s7 import ArtyS7_25Platform, ArtyS7_50Platform
from .cmod_s7 import CmodS7_Platform

__all__ = (
	'ArtyS7_25Platform',
	'ArtyS7_50Platform',
	'CmodS7_Platform',
)



# SPDX-License-Identifier: BSD-2-Clause

import warnings

__all__ = (
	'_deprecated_import',
)

def _deprecated_import(old: str, new: str):
	warnings.warn(
		f'The import from {old} has been deprecated and moved to {new}',
		DeprecationWarning, stacklevel = 3
	)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                        import Attrs, Clock, Connector, Pins, Resource, Subsignal
from torii.build.run                    import BuildProducts
from torii.platform.resources.display   import Display7SegResource, VGAResource
from torii.platform.resources.interface import PS2Resource, UARTResource
from torii.platform.resources.memory    import NORFlashResources, SDCardResources, SDRAMResource
from torii.platform.resources.user      import ButtonResources, LEDResources, SwitchResources
from torii.platform.vendor.altera       import AlteraPlatform

__all__ = (
	'DE0Platform',
)

class DE0Platform(AlteraPlatform):
	device      = 'EP3C16' # Cyclone III 15K LEs
	package     = 'F484'   # FBGA-484
	speed       = 'C6'
	default_clk = 'clk50'

	pretty_name = 'DE0'
	description = 'terasIC DE0 Altera Cyclone III Development Board'

	resources   = [
		Resource(
			'clk50', 0, Pins('G21', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'clk50', 1, Pins('B12', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),

		*LEDResources(
			pins = 'J1 J2 J3 H1 F2 E1 C1 C2 B2 B1',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		*ButtonResources(
			pins = 'H2 G3 F1', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		*SwitchResources(
			pins = 'J6 H5 H6 G4 G5 J7 H7 E3 E4 D2',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(
			0,
			a = 'E11', b = 'F11', c = 'H12', d = 'H13', e = 'G12', f = 'F12', g = 'F13', dp = 'D13', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(
			1,
			a = 'A13', b = 'B13', c = 'C13', d = 'A14', e = 'B14', f = 'E14', g = 'A15', dp = 'B15', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(
			2,
			a = 'D15', b = 'A16', c = 'B16', d = 'E15', e = 'A17', f = 'B17', g = 'F14', dp = 'A18', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(
			3,
			a = 'B18', b = 'F15', c = 'A19', d = 'B19', e = 'C19', f = 'D19', g = 'G15', dp = 'G16', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		UARTResource(
			0,
			rx = 'U22', tx = 'U21', rts = 'V22', cts = 'V21',
			attrs = Attrs(io_standard = '3.3-V LVTTL'),
			role = 'dce'
		),
		Resource(
			'display_hd44780', 0,
			Subsignal('e', Pins('E21', dir = 'o')),
			Subsignal('d', Pins('D22 D21 C22 C21 B22 B21 D20 C20', dir = 'io')),
			Subsignal('rw', Pins('E22', dir = 'o')),
			Subsignal('rs', Pins('F22', dir = 'o')),
			# Backlight
			Subsignal('bl', Pins('F21', dir = 'o')),
			Attrs(io_standard = '3.3-V LVTTL')
		),
		VGAResource(
			0,
			r = 'H19 H17 H20 H21',
			g = 'H22 J17 K17 J21',
			b = 'K22 K21 J22 K18',
			hs = 'L21', vs = 'L22',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		PS2Resource(
			0, # Keyboard
			clk = 'P22', dat = 'P21', attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		PS2Resource(
			1, # Mouse
			clk = 'R21', dat = 'R22', attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		*SDCardResources(
			0,
			clk = 'Y21', cmd = 'Y22', dat0 = 'AA22', dat3 = 'W21', wp_n = 'W20',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		SDRAMResource(
			0,
			clk = 'E5', cke = 'E6', cs_n = 'G7', we_n = 'D6', ras_n = 'F7', cas_n = 'G8',
			ba = 'B5 A4', a = 'C4 A3 B3 C3 A5 C6 B6 A6 C7 B7 B4 A7 C8',
			dq = 'D10 G10 H10 E9 F9 G9 H9 F8 A8 B9 A9 C10 B10 A10 E10 F10', dqm = 'E7 B8',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		*NORFlashResources(
			0,
			rst = 'R1', byte_n = 'AA1',
			cs_n = 'N8', oe_n = 'R6', we_n = 'P4', wp_n = 'T3', by = 'M7',
			a = 'P7 P5 P6 N7 N5 N6 M8 M4 P2 N2 N1 M3 M2 M1 L7 L6 AA2 M5 M6 P1 P3 R2',
			dq = 'R7 P8 R8 U1 V2 V3 W1 Y1 T5 T7 T4 U2 V1 V4 W2 Y2',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
	]
	connectors  = [
		Connector(
			'j', 4,
			'AB12 AB16 AA12 AA16 AA15 AB15 AA14 AB14 AB13 AA13 -    -    '
			'AB10 AA10 AB8  AA8  AB5  AA5  AB3  AB4  AA3  AA4  V14  U14  '
			'Y13  W13  U13  V12  -    -    R10  V11  Y10  W10  T8   V8   '
			'W7   W6   V5   U7   -    -    -    -    -    -    -    -    '
		),
		Connector(
			'j', 5,
			'AB11 AA20 AA11 AB20 AA19 AB19 AB18 AA18 AA17 AB17 -    -    '
			'Y17  W17  U15  T15  W15  V15  R16  AB9  T16  AA9  AA7  AB7  '
			'T14  R14  U12  T12  -    -    R11  R12  U10  T10  U9   T9   '
			'Y7   U8   V6   V7   -    -    -    -    -    -    -    -    '
		),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		quartus_pgm = environ.get('QUARTUS_PGM', 'quartus_pgm')
		with products.extract(f'{name}.sof') as bitstream_filename:
			check_call([
				quartus_pgm, '--haltcc', '--mode', 'JTAG',
				'--operation', 'P;' + bitstream_filename
			])


if __name__ == '__main__':
	from ...test.blinky import Blinky
	DE0Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

import subprocess

from torii.build                        import Attrs, Clock, Connector, Pins, Resource
from torii.platform.resources.interface import UARTResource
from torii.platform.resources.memory    import SPIFlashResources
from torii.platform.resources.user      import ButtonResources, LEDResources, RGBLEDResource
from torii.platform.vendor.xilinx       import XilinxPlatform

'''
Example Usage:
	platform = CModS7_Platform(toolchain = 'Symbiflow')
	platform.build(Top(), do_program = True)

Supported programmer:
	openFPGALoader
'''

__all__ = (
	'CmodS7_Platform',
)

class CmodS7_Platform(XilinxPlatform):
	device      = 'xc7s25'
	package     = 'csga225'
	speed       = '1'
	default_clk = 'clk12'

	pretty_name = 'Cmod S7'
	description = 'Digilent Cmod S7 Xilinx Spartan7-25 FPGA Module'

	resources   = [
		Resource(
			'clk12', 0, Pins('M9', dir = 'i'), Clock(12e6), Attrs(IOSTANDARD = 'LVCMOS33')
		),
		*LEDResources(pins = 'E2 K1 J1 E1', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		RGBLEDResource(
			0,
			r = 'F2', g = 'D3', b = 'F1', invert = True,
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		*ButtonResources(pins = 'D2 D1', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		UARTResource(0, rx = 'L12', tx = 'K15', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		# Clock only via STARTUPE2 primitive
		*SPIFlashResources(
			0,
			cs_n = 'L11', clk = 'F5', copi = 'H14', cipo = 'H15', wp_n = 'J12',
			hold_n = 'K13',
			attrs = Attrs(IOSTANDARD='LVCMOS33')
		),
		# One-wire interface to crypto authentication device
		# May not be populated on the board
		Resource('atsha204a', 0, Pins('D17', dir = 'io'), Attrs(IOSTANDARD = 'LVCMOS33'))
	]

	connectors  = [
		Connector('pmod', 0, 'J2 H2 H4 F3 - - H3 H1 G1 F4 - -'), # JA
		# Pin 24/25 are VCC and GND
		# Pin 32/33 are analog (XADC)
		# Pin 9-15 and 34-39 do not exist
		Connector(
			'gpio', 0,
			'L1  M4  M3  N2  M2  P3  N3  P1  N1  -   -   -  '
			'-   -   -   P14 P15 N13 N15 N14 M15 M14 L15 -  '
			'-   L14 K14 J15 L13 M13 J11 -   -   -   -   -  '
			'-   -   -   C5  A2  B2  B1  C1  B3  B4  A3  A4 '
		),
		Connector('xadc', 0, {
			'vaux5_n': 'A13',
			'vaux5_p': 'A14',
			'vaux12_n': 'A11',
			'vaux12_p': 'A12'
		})
	]

	def toolchain_program(self, products, name):
		with products.extract(f'{name}.bit') as bitstream_filename:
			subprocess.check_call([
				'openFPGALoader', '-c', 'digilent', '--fpga-part', self.device, bitstream_filename
			])

if __name__ == '__main__':
	from ...test.blinky import Blinky
	CmodS7_Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from .alchitry_au     import AlchitryAuPlatform
from .arty_a7         import ArtyA7_35Platform, ArtyA7_100Platform
from .cmod_a7         import CmodA7_15Platform, CmodA7_35Platform
from .mega65          import Mega65r3Platform
from .nexys4ddr       import Nexys4DDRPlatform
from .te0714_03_50_2I import TE0714_03_50_2IPlatform

__all__ = (
	'AlchitryAuPlatform',
	'ArtyA7_35Platform',
	'ArtyA7_100Platform',
	'CmodA7_15Platform',
	'CmodA7_35Platform',
	'Mega65r3Platform',
	'Nexys4DDRPlatform',
	'TE0714_03_50_2IPlatform',
)



# SPDX-License-Identifier: BSD-2-Clause

from typing                             import Literal

from torii.build                        import Attrs, Clock, Connector, DiffPairs, Pins, PinsN, Resource, Subsignal
from torii.build.run                    import BuildProducts
from torii.platform.resources.interface import PS2Resource, UARTResource
from torii.platform.resources.memory    import SPIFlashResources
from torii.platform.resources.user      import ButtonResources, LEDResources, SwitchResources
from torii.platform.vendor.xilinx       import XilinxPlatform

__all__ = (
	'AtlysPlatform',
)

class AtlysPlatform(XilinxPlatform):
	'''Platform file for Digilent Atlys Spartan 6 board.
	https://digilent.com/reference/programmable-logic/atlys/start'''

	device  = 'xc6slx45'
	package = 'csg324'
	speed   = '3'

	pretty_name = 'Atlys'
	description = 'Digilent Atlys Xilinx Spartan 6 Trainer Board'

	def __init__(self, *, JP12: Literal['2V5', '3V3'] = '2V5', **kwargs) -> None:
		super().__init__(**kwargs)

		if JP12 not in ('2V5', '3V3'):
			raise ValueError(f'JP12 must be either \'2V5\' or \'3V3\' not {JP12!r}')
		self._JP12 = JP12

	def bank2_iostandard(self) -> str:
		return 'LVCMOS25' if self._JP12 == '2V5' else 'LVCMOS33'

	default_clk = 'clk100'
	default_rst = 'rst'

	resources   = [
		Resource('rst', 0, PinsN('T15', dir = 'i'), Attrs(IOSTANDARD = bank2_iostandard)), # RESET
		Resource(
			'clk100', 0, Pins('L15', dir = 'i'), Clock(100e6), Attrs(IOSTANDARD = 'LVCMOS33')
		), # GCLK
		*LEDResources(pins = 'U18 M14 N14 L14 M13 D3 P16', attrs = Attrs(IOSTANDARD = 'LVCMOS33')), # LD0..6
		Resource('led', 7, Pins('N12', dir = 'o'), Attrs(IOSTANDARD = bank2_iostandard)),    # LD7
		# BTNU/BTNL/BTND/BTNR/BTNC
		*ButtonResources(pins = 'N4 P4 P3 F6 F5', attrs = Attrs(IOSTANDARD = 'LVCMOS18')),
		*SwitchResources(
			pins = {
				0: 'A10', 1: 'D14', 2: 'C14', 3: 'P15', 7: 'E4',
			}, attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		), # SW0..3,SW7
		*SwitchResources(
			pins = {
				4: 'P12', 5: 'R5', 6: 'T5',
			}, attrs = Attrs(IOSTANDARD = bank2_iostandard)
		), # SW4..6
		UARTResource(0, rx = 'A16', tx = 'B16', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),     # J17/UART
		PS2Resource(
			0, # PS/2 keyboard interface converted from J13 'HOST' USB connector
			clk = 'P17', dat = 'N15', attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		PS2Resource(
			1, # PS/2 mouse interface converted from J13 'HOST' USB connector
			clk = 'N18', dat = 'P18', attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		*SPIFlashResources(
			0,
			cs_n = 'AE14', clk = 'AH18', copi = 'AF14', cipo = 'AF20', wp_n = 'AG21', hold_n = 'AG17',
			attrs = Attrs(IOSTANDARD = 'LVCMOS25', SLEW = 'FAST'),
		),
		# TODO(aki): Replace with `DDR2Resource` when more granular attrs are possible
		Resource(
			'ddr2', 0,
			Subsignal(
				'clk', DiffPairs('G3', 'G1', dir = 'o'), Attrs(IOSTANDARD = 'DIFF_SSTL18_II', IN_TERM = 'NONE')
			),
			Subsignal('clk_en', Pins('H7', dir = 'o')),
			Subsignal('we', PinsN('E3', dir = 'o')),
			Subsignal('ras', PinsN('L5', dir = 'o')),
			Subsignal('cas', PinsN('K5', dir = 'o')),
			Subsignal('a', Pins('J7 J6 H5 L7 F3 H4 H3 H6 D2 D1 F4 D3 G6', dir = 'o')),
			Subsignal('ba', Pins('F2 F1 E1', dir = 'o')),
			Subsignal('dqs', DiffPairs('P2 L4', 'P1 L3', dir = 'o'), Attrs(IOSTANDARD = 'DIFF_SSTL18_II')),
			Subsignal('dq', Pins('L2 L1 K2 K1 H2 H1 J3 J1 M3 M1 N2 N1 T2 T1 U2 U1', dir = 'io')),
			Subsignal('dm', Pins('K4 K3', dir = 'o')),
			Subsignal('odt', Pins('K6', dir = 'o')),
			Attrs(IOSTANDARD = 'SSTL18_II', SLEW = 'FAST'),
		),
		# TODO(aki): Replace with `EthernetResource` when more granular attrs are possible
		Resource(
			'eth_gmii', 0,
			Subsignal('rst', PinsN('G13', dir = 'o')),
			Subsignal('int', PinsN('L16', dir = 'o')),
			Subsignal('mdio', Pins('N17', dir = 'io')),
			Subsignal('mdc', Pins('F16', dir = 'o')), # Max 8.3MHz
			Subsignal('gtx_clk', Pins('L12', dir = 'o')),
			Subsignal('tx_clk', Pins('K16', dir = 'i')),
			Subsignal('tx_en', Pins('H15', dir = 'o')),
			Subsignal('tx_er', Pins('G18', dir = 'o')),
			Subsignal('tx_data', Pins('H16 H13 K14 K13 J13 G14 H12 K12', dir = 'o')),
			Subsignal('rx_clk', Pins('K15', dir = 'i')),
			Subsignal('rx_dv', Pins('F17', dir = 'i'), Attrs(PULLDOWN = 'TRUE')),
			Subsignal('rx_er', Pins('F18', dir = 'i')),
			Subsignal('rx_data', Pins('G16 H14 E16 F15 F14 E18 D16 D17', dir = 'i')),
			Subsignal('col', Pins('C17', dir = 'i')),
			Subsignal('crs', Pins('C18', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
		# TODO(aki): Replace with `EthernetResource` when more granular attrs are possible
		Resource(
			'eth_rgmii', 0,
			Subsignal('rst', PinsN('G13', dir = 'o')),
			Subsignal('int', PinsN('L16', dir = 'o')),
			Subsignal('mdio', Pins('N17', dir = 'io')),
			Subsignal('mdc', Pins('F16', dir = 'o')), # Max 8.3MHz
			Subsignal('tx_clk', Pins('L12', dir = 'o')),
			Subsignal('tx_ctl', Pins('H15', dir = 'o')),
			Subsignal('tx_data', Pins('H16 H13 K14 K13', dir = 'o')),
			Subsignal('rx_clk', Pins('K15', dir = 'i')),
			Subsignal('rx_ctl', Pins('F17', dir = 'i'), Attrs(PULLDOWN = 'TRUE')),
			Subsignal('rx_data', Pins('G16 H14 E16 F15', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
		# TODO(aki): Replace with `EthernetResource` when more granular attrs are possible
		Resource(
			'eth_mii', 0,
			Subsignal('rst', PinsN('G13', dir = 'o')),
			Subsignal('int', PinsN('L16', dir = 'o')),
			Subsignal('mdio', Pins('N17', dir = 'io')),
			Subsignal('mdc', Pins('F16', dir = 'o')), # Max 8.3MHz
			Subsignal('tx_clk', Pins('K16', dir = 'i')),
			Subsignal('tx_en', Pins('H15', dir = 'o')),
			Subsignal('tx_er', Pins('G18', dir = 'o')),
			Subsignal('tx_data', Pins('H16 H13 K14 K13', dir = 'o')),
			Subsignal('rx_clk', Pins('K15', dir = 'i')),
			Subsignal('rx_dv', Pins('F17', dir = 'i'), Attrs(PULLDOWN = 'TRUE')),
			Subsignal('rx_er', Pins('F18', dir = 'i')),
			Subsignal('rx_data', Pins('G16 H14 E16 F15', dir = 'i')),
			Subsignal('col', Pins('C17', dir = 'i')),
			Subsignal('crs', Pins('C18', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
		# Device does not support RMII
		# TODO(aki): Replace with `EthernetResource` when TBI/RTBI support merged + released in Torii
		Resource(
			'eth_tbi', 0,
			Subsignal('rst', PinsN('G13', dir = 'o')),
			Subsignal('int', PinsN('L16', dir = 'o')),
			Subsignal('mdio', Pins('N17', dir = 'io')),
			Subsignal('mdc', Pins('F16', dir = 'o')), # Max 8.3MHz
			Subsignal('tx_clk', Pins('L12', dir = 'o')),
			Subsignal('tx_data', Pins('H16 H13 K14 K13 J13 G14 H12 K12 H15 G18', dir = 'o')),
			Subsignal('rx_clk', Pins('K15 L12', dir = 'i')),
			Subsignal('rx_data', Pins('G16 H14 E16 F15 F14 E18 D16 D17 F17 F18', dir = 'i')),
			Subsignal('lpbk', Pins('C17', dir = 'o'), Attrs(PULLDOWN = 'TRUE')),
			Subsignal('comma', Pins('C18', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
		# TODO(aki): Replace with `EthernetResource` when TBI/RTBI support merged + released in Torii
		Resource(
			'eth_rtbi', 0,
			Subsignal('rst', PinsN('G13', dir = 'o')),
			Subsignal('int', PinsN('L16', dir = 'o')),
			Subsignal('mdio', Pins('N17', dir = 'io')),
			Subsignal('mdc', Pins('F16', dir = 'o')), # Max 8.3MHz
			Subsignal('tx_clk', Pins('L12', dir = 'o')),
			Subsignal('tx_data', Pins('H16 H13 K14 K13 H15', dir = 'o')),
			Subsignal('rx_clk', Pins('K15', dir = 'i')),
			Subsignal('rx_data', Pins('G16 H14 E16 F15 F17', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
		# TODO(aki): Replace with `HDMIResource` when merged + released in Torii
		Resource(
			'hdmi', 0, # J1, input only due to on board buffer, HDMI A connector
			Subsignal('scl',  Pins('C13'), Attrs(IOSTANDARD = 'I2C')),
			Subsignal('sda',  Pins('A13'), Attrs(IOSTANDARD = 'I2C')),
			Subsignal('clk', DiffPairs('D11', 'C11', dir = 'i')),
			Subsignal('d', DiffPairs('G9 B11 B12', 'F9 A11 A12', dir = 'i')),
			Attrs(IOSTANDARD = 'TMDS_33'),
		),
		# TODO(aki): Replace with `HDMIResource` when merged + released in Torii
		Resource(
			'hdmi', 1, # J2, output only due to on board buffer, HDMI A connector
			Subsignal('scl', Pins('D9'), Attrs(IOSTANDARD = 'I2C')),
			Subsignal('sda', Pins('C9'), Attrs(IOSTANDARD = 'I2C')),
			Subsignal('clk', DiffPairs('B6', 'A6', dir = 'o')),
			Subsignal('d', DiffPairs('D8 C7 B8', 'C8 A7 A8', dir = 'o')),
			Attrs(IOSTANDARD = 'TMDS_33'),
		),
		# TODO(aki): Replace with `HDMIResource` when merged + released in Torii
		Resource(
			'hdmi', 2, # J3, input only due to on board buffer, HDMI A connector
			Subsignal('scl', Pins('M16'), Attrs(IOSTANDARD = 'I2C')),
			Subsignal('sda', Pins('M18'), Attrs(IOSTANDARD = 'I2C')),
			Subsignal('clk', DiffPairs('H17', 'H18', dir = 'i')),
			Subsignal('d', DiffPairs('K17 L17 J16', 'K18 L18 J18', dir = 'i')),
			Attrs(IOSTANDARD = 'TMDS_33'),
		),
		# TODO(aki): Replace with `HDMIResource` when merged + released in Torii
		Resource(
			'hdmi', 3, # JA, input/output as it is unbuffered, HDMI D connector
			Subsignal('scl', Pins('C13'), Attrs(IOSTANDARD = 'I2C')),
			Subsignal('sda', Pins('A13'), Attrs(IOSTANDARD = 'I2C')),
			Subsignal('clk', DiffPairs('T9', 'V9')),
			Subsignal('d', DiffPairs('R3 T4 N5', 'T3 V4 P6')),
			Attrs(IOSTANDARD = 'TMDS_33'),
		),
		# TODO(aki): Replace with `AC97Resource` when merged + released in Torii
		Resource(
			'ac97', 0,
			Subsignal('clk', Pins('L13', dir = 'o')),
			Subsignal('sync', Pins('U17', dir = 'o')),
			Subsignal('reset', Pins('T17', dir = 'o')),
			Subsignal('sdo', Pins('N18', dir = 'o')),
			Subsignal('sdi', Pins('T18', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
	]
	connectors  = [
		Connector('pmod', 0, 'T3 R3 P6 N5 - - V9 T9 V4 T4 - -'), # JB
		Connector(
			'vhdci', 0, # JC
			'U16 - U15 U13 - M11 R11 - T12 N10 - M10 U11 - R10 - - - - U10 - R8  M8  - U8  U7  - N7  T6  - R7  N6  - U5'
			'V16 - V15 V13 - N11 T11 - V12 P11 - N9  V11 - T10 - - - - V10 - T8  N8  - V8  V7  - P8  V6  - T7  P7  - V5'
		),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from subprocess import run
		from textwrap import dedent

		from torii.tools import require_tool

		impact = require_tool('impact')

		with products.extract(f'{name}.bit') as bitstream:
			cmd = dedent(f'''
				setMode -bscan
				setCable -port auto
				addDevice -p 1 -file '{bitstream}'
				program -p 1
				exit
			''').encode('utf-8')
			run([impact, '-batch'], input = cmd, check = True)


if __name__ == '__main__':
	from ...test.blinky import Blinky
	AtlysPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                        import Attrs, Clock, Connector, DiffPairs, Pins, PinsN, Resource, Subsignal
from torii.build.run                    import BuildPlan, BuildProducts
from torii.hdl.ir                       import Fragment
from torii.platform.resources.interface import DirectUSBResource, SPIResource, UARTResource
from torii.platform.resources.memory    import SDCardResources, SDRAMResource
from torii.platform.resources.user      import ButtonResources, LEDResources
from torii.platform.vendor.lattice.ecp5 import ECP5Platform

__all__ = (
	'ULX3S_12F_Platform',
	'ULX3S_25F_Platform',
	'ULX3S_45F_Platform',
	'ULX3S_85F_Platform',
)

class _ULX3SPlatform(ECP5Platform):
	package     = 'BG381'
	speed       = '6'
	default_clk = 'clk25'

	resources   = [
		Resource('clk25', 0, Pins('G2', dir = 'i'), Clock(25e6), Attrs(IO_TYPE = 'LVCMOS33')),
		# Used to reload FPGA configuration.
		Resource('program', 0, PinsN('M4', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')),
		*LEDResources(
			pins = 'B2 C2 C1 D2 D1 E2 E1 H3',
			attrs = Attrs(IO_TYPE = 'LVCMOS33', DRIVE = '4')
		),
		*ButtonResources(
			pins = 'R1 T1 R18 V1 U1 H16',
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'DOWN')
		),
		*ButtonResources(
			'switch',
			pins = 'E8 D8 D7 E7',
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'DOWN')
		),
		# Semantic aliases by button label.
		Resource('button_pwr', 0, PinsN('D6', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')),
		Resource('button_fire', 0, Pins('R1', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'DOWN')),
		Resource('button_fire', 1, Pins('T1', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'DOWN')),
		Resource('button_up', 0, Pins('R18', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'DOWN')),
		Resource('button_down', 0, Pins('V1', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'DOWN')),
		Resource('button_left', 0, Pins('U1', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'DOWN')),
		Resource('button_right', 0, Pins('H16', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'DOWN')),
		# FTDI connection.
		UARTResource(
			0,
			rx = 'M1', tx = 'L4', rts = 'M3', dtr = 'N1', role = 'dce',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),
		Resource('uart_tx_enable', 0, Pins('L3', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS33')),
		*SDCardResources(
			0,
			clk = 'J1', cmd = 'J3', dat0 = 'K2', dat1 = 'K1', dat2 = 'H2', dat3 = 'H1',
			attrs = Attrs(IO_TYPE = 'LVCMOS33', SLEW = 'FAST')
		),
		# XXX(aki):
		# We can't replace this with `SPIFlashResource` because it needs a `clk` signal
		# And the SPI Clock needs to be done via the `USRMCLK` block in the ECP5.
		Resource(
			'spi_flash', 0,
			Subsignal('cs', PinsN('R2', dir = 'o')),
			Subsignal('copi', Pins('W2', dir = 'o')),
			Subsignal('cipo', Pins('V2', dir = 'i')),
			Subsignal('hold', PinsN('W1', dir = 'o')),
			Subsignal('wp', PinsN('Y2', dir = 'o')),
			Attrs(PULLMODE = 'NONE', DRIVE = '4', IO_TYPE = 'LVCMOS33')
		),
		SDRAMResource(
			0,
			clk = 'F19', cke = 'F20', cs_n = 'P20', we_n = 'T20', cas_n = 'T19', ras_n = 'R20', dqm = 'U19 E20',
			ba = 'P19 N20', a = 'M20 M19 L20 L19 K20 K19 K18 J20 J19 H20 N19 G20 G19',
			dq = 'J16 L18 M18 N18 P18 T18 T17 U20 E19 D20 D19 C20 E18 F18 J18 J17',
			attrs = Attrs(PULLMODE = 'NONE', DRIVE = '4', SLEWRATE = 'FAST', IO_TYPE = 'LVCMOS33')
		),
		# SPI bus for ADC.
		SPIResource(
			'adc', 0,
			cs_n = 'R17', copi = 'R16', cipo = 'U16', clk = 'P17',
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')
		),
		# TRRS audio jack
		Resource(
			'audio', 0,
			Subsignal('l', Pins('E4 D3 C3 B3', dir = 'o')),
			Subsignal('r', Pins('A3 B5 D5 C5', dir = 'o')),
			Subsignal('ring2', Pins('H5 F2 F5 E5', dir = 'o')), # extra ring out for video adapters
		),
		# ESP-32 connections
		Resource(
			'esp32', 0,
			Subsignal('en', Pins('F1', dir = 'o'), Attrs(PULLMODE = 'UP')),
			Subsignal('tx', Pins('K3', dir = 'o'), Attrs(PULLMODE = 'UP')),
			Subsignal('rx', Pins('K4', dir = 'i'), Attrs(PULLMODE = 'UP')),
			Subsignal('gpio0', Pins('L2'), Attrs(PULLMODE = 'UP')),
			Subsignal('gpio5', Pins('N4')),
			Subsignal('gpio16', Pins('L1'), Attrs(PULLMODE = 'UP')),
			Subsignal('gpio17', Pins('N3'), Attrs(PULLMODE = 'UP')),
			Attrs(IO_TYPE = 'LVCMOS33', DRIVE = '4')
		),
		# PCB antenna, tuned to 433MHz
		Resource('ant', 0, Pins('G1', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS33')),
		# Differential versions of our connector I/O.
		Resource('diff_gpio', 0, DiffPairs('B11', 'C11'), Attrs(IO_TYPE = 'LVCMOS33')),
		Resource('diff_gpio', 1, DiffPairs('A10', 'A11'), Attrs(IO_TYPE = 'LVCMOS33')),
		Resource('diff_gpio', 2, DiffPairs('A9', 'B10'), Attrs(IO_TYPE = 'LVCMOS33')),
		Resource('diff_gpio', 3, DiffPairs('B9', 'C10'),  Attrs(IO_TYPE = 'LVCMOS33')),
		# HDMI (only TX, due to the top bank of ECP5 only supporting diff. outputs)
		# TODO(aki): Replace with `HDMIResource` when merged + released in Torii
		Resource(
			'hdmi', 0,
			Subsignal(
				'cec', Pins('A18', dir = 'io'),
				Attrs(IO_TYPE = 'LVCMOS33', DRIVE = '4', PULLMODE = 'UP')
			),
			Subsignal(
				'clk', DiffPairs('A17', 'B18', dir = 'o'),
				Attrs(IO_TYPE = 'LVCMOS33D', DRIVE = '4')
			),
			Subsignal(
				'd',   DiffPairs('A16 A14 A12', 'B16 C14 A13', dir = 'o'),
				Attrs(IO_TYPE = 'LVCMOS33D', DRIVE = '4')
			),
			Subsignal(
				'eth', DiffPairs('A19', 'B20', dir = 'o'),
				Attrs(IO_TYPE = 'LVCMOS33D', DRIVE = '4')
			),
			Subsignal(
				'scl', Pins('E12', dir = 'io'),
				Attrs(IO_TYPE = 'LVCMOS33', DRIVE = '4', PULLMODE = 'UP')
			),
			Subsignal(
				'sda', Pins('B19', dir = 'io'),
				Attrs(IO_TYPE = 'LVCMOS33', DRIVE = '4', PULLMODE = 'UP')
			)
		),
		DirectUSBResource(
			0,
			d_p = 'D15', d_n = 'E15', pullup = 'B12',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		)
	]

	connectors = [
		Connector('gpio', 0, {
			'0+': 'B11', '0-': 'C11', '1+': 'A10', '1-': 'A11',
			'2+': 'A9', '2-': 'B10', '3+': 'B9', '3-': 'C10',
			'4+': 'A7', '4-': 'A8', '5+': 'C8', '5-': 'B8',
			'6+': 'C6', '6-': 'C7', '7+': 'A6', '7-': 'B6',
			'8+': 'A4', '8-': 'A5', '9+': 'A2', '9-': 'B1',
			'10+': 'C4', '10-': 'B4', '11+': 'F4', '11-': 'E3',
			'12+': 'G3', '12-': 'F3', '13+': 'H4', '13-': 'G5',
			'14+': 'U18', '14-': 'U17', '15+': 'N17', '15-': 'P16',
			'16+': 'N16', '16-': 'M17', '17+': 'L16', '17-': 'L17',
			'18+': 'H18', '18-': 'H17', '19+': 'F17', '19-': 'G18',
			'20+': 'D18', '20-': 'E17', '21+': 'C18', '21-': 'D17',
			'22+': 'B15', '22-': 'C15', '23+': 'B17', '23-': 'C17',
			'24+': 'C16', '24-': 'D16', '25+': 'D14', '25-': 'E14',
			'26+': 'B13', '26-': 'C13', '27+': 'D13', '27-': 'E13',
		})
	]

	@property
	def required_tools(self) -> list[str]:
		return super().required_tools + [
			'openFPGALoader'
		]

	def toolchain_prepare(self, fragment: Fragment, name: str, **kwargs) -> BuildPlan:
		overrides = dict(ecppack_opts = '--compress')
		overrides.update(kwargs)
		return super().toolchain_prepare(fragment, name, **overrides)

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		tool = environ.get('OPENFPGALOADER', 'openFPGALoader')
		with products.extract(f'{name}.bit') as bitstream_filename:
			check_call([tool, '-b', 'ulx3s', '-m', bitstream_filename])


class ULX3S_12F_Platform(_ULX3SPlatform):
	device = 'LFE5U-12F'

	pretty_name = 'ULXS3-12F'
	description = 'Radiona ULXS3-12F Lattice ECP5-12F Development Board'

class ULX3S_25F_Platform(_ULX3SPlatform):
	device = 'LFE5U-25F'

	pretty_name = 'ULXS3-24F'
	description = 'Radiona ULXS3-25F Lattice ECP5-25F Development Board'

class ULX3S_45F_Platform(_ULX3SPlatform):
	device = 'LFE5U-45F'

	pretty_name = 'ULXS3-45F'
	description = 'Radiona ULXS3-45F Lattice ECP5-45F Development Board'

class ULX3S_85F_Platform(_ULX3SPlatform):
	device = 'LFE5U-85F'

	pretty_name = 'ULXS3-85F'
	description = 'Radiona ULXS3-85F Lattice ECP5-85F Development Board'

if __name__ == '__main__':
	from argparse      import ArgumentParser

	from ...test.blinky import Blinky

	variants = {
		'12F': ULX3S_12F_Platform,
		'25F': ULX3S_25F_Platform,
		'45F': ULX3S_45F_Platform,
		'85F': ULX3S_85F_Platform
	}

	# Figure out which FPGA variant we want to target...
	parser = ArgumentParser()
	parser.add_argument('variant', choices = variants.keys())
	args = parser.parse_args()

	# ... and run Blinky on it.
	platform = variants[args.variant]
	platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from textwrap                           import dedent

from torii.build                        import Attrs, Clock, Connector, DiffPairs, Pins, PinsN, Resource, Subsignal
from torii.build.run                    import BuildProducts
from torii.platform.resources.interface import UARTResource
from torii.platform.resources.memory    import SPIFlashResources
from torii.platform.resources.user      import LEDResources, SwitchResources
from torii.platform.vendor.lattice.ecp5 import ECP5Platform

__all__ = (
	'VersaECP5Platform',
)

class VersaECP5Platform(ECP5Platform):
	device      = 'LFE5UM-45F'
	package     = 'BG381'
	speed       = '8'
	default_clk = 'clk100'
	default_rst = 'rst'

	pretty_name = 'Versa'
	description = 'Lattice Versa ECP5-45F Evaluation Board'

	resources   = [
		Resource('rst', 0, PinsN('T1', dir = 'i'), Attrs(IO_TYPE = 'LVCMOS33')),
		Resource('clk100', 0, DiffPairs('P3', 'P4', dir = 'i'), Clock(100e6), Attrs(IO_TYPE = 'LVDS')),
		Resource('pclk', 0, DiffPairs('A4', 'A5', dir = 'i'), Attrs(IO_TYPE = 'LVDS')),
		*LEDResources(
			pins = 'E16 D17 D18 E18 F17 F18 E17 F16', invert = True,
			attrs = Attrs(IO_TYPE = 'LVCMOS25')
		),
		Resource(
			'alnum_led', 0,
			Subsignal('a', PinsN('M20', dir = 'o')),
			Subsignal('b', PinsN('L18', dir = 'o')),
			Subsignal('c', PinsN('M19', dir = 'o')),
			Subsignal('d', PinsN('L16', dir = 'o')),
			Subsignal('e', PinsN('L17', dir = 'o')),
			Subsignal('f', PinsN('M18', dir = 'o')),
			Subsignal('g', PinsN('N16', dir = 'o')),
			Subsignal('h', PinsN('M17', dir = 'o')),
			Subsignal('j', PinsN('N18', dir = 'o')),
			Subsignal('k', PinsN('P17', dir = 'o')),
			Subsignal('l', PinsN('N17', dir = 'o')),
			Subsignal('m', PinsN('P16', dir = 'o')),
			Subsignal('n', PinsN('R16', dir = 'o')),
			Subsignal('p', PinsN('R17', dir = 'o')),
			Subsignal('dp', PinsN('U1', dir = 'o')),
			Attrs(IO_TYPE = 'LVCMOS25')
		),
		*SwitchResources(
			pins = {0: 'H2',  1: 'K3',  2: 'G3',  3: 'F2' },
			attrs = Attrs(IO_TYPE = 'LVCMOS15')
		),
		*SwitchResources(
			pins = {4: 'J18', 5: 'K18', 6: 'K19', 7: 'K20'},
			attrs = Attrs(IO_TYPE = 'LVCMOS25')
		),
		UARTResource(
			0,
			rx = 'C11', tx = 'A11',
			attrs = Attrs(IO_TYPE = 'LVCMOS33', PULLMODE = 'UP')
		),
		*SPIFlashResources(
			0,
			cs_n = 'R2', clk = 'U3', cipo = 'W2', copi = 'V2', wp_n = 'Y2', hold_n = 'W1',
			attrs = Attrs(IO_TYPE = 'LVCMOS33')
		),
		Resource(
			'eth_clk125', 0, Pins('L19', dir = 'i'), Clock(125e6), Attrs(IO_TYPE = 'LVCMOS25')
		),
		Resource(
			'eth_clk125_pll', 0, Pins('U16', dir = 'i'), Clock(125e6), Attrs(IO_TYPE = 'LVCMOS25')
		), # NC by default
		# TODO(aki): Replace with `EthernetResource` when it has `rst` signal support
		Resource(
			'eth_rgmii', 0,
			Subsignal('rst', PinsN('U17', dir = 'o')),
			Subsignal('mdc', Pins('T18', dir = 'o')),
			Subsignal('mdio', Pins('U18', dir = 'io')),
			Subsignal('tx_clk', Pins('P19', dir = 'o')),
			Subsignal('tx_ctl', Pins('R20', dir = 'o')),
			Subsignal('tx_data', Pins('N19 N20 P18 P20', dir = 'o')),
			Subsignal('rx_clk', Pins('L20', dir = 'i')),
			Subsignal('rx_ctl', Pins('U19', dir = 'i')),
			Subsignal('rx_data', Pins('T20 U20 T19 R18', dir = 'i')),
			Attrs(IO_TYPE = 'LVCMOS25')
		),
		# TODO(aki): Replace with `EthernetResource` when it has SGMII support
		Resource(
			'eth_sgmii', 0,
			Subsignal('rst', PinsN('U17', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS25')),
			Subsignal('mdc', Pins('T18', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS25')),
			Subsignal('mdio', Pins('U18', dir = 'io'), Attrs(IO_TYPE = 'LVCMOS25')),
			Subsignal('tx', DiffPairs('W13', 'W14', dir = 'o')),
			Subsignal('rx', DiffPairs('Y14', 'Y15', dir = 'i')),
		),
		Resource(
			'eth_clk125', 1, Pins('J20', dir = 'i'), Clock(125e6), Attrs(IO_TYPE = 'LVCMOS25')
		),
		Resource(
			'eth_clk125_pll', 1, Pins('C18', dir = 'i'), Clock(125e6), Attrs(IO_TYPE = 'LVCMOS25')
		), # NC by default
		# TODO(aki): Replace with `EthernetResource` when it has `rst` signal support
		Resource(
			'eth_rgmii', 1,
			Subsignal('rst', PinsN('F20', dir = 'o')),
			Subsignal('mdc', Pins('G19', dir = 'o')),
			Subsignal('mdio', Pins('H20', dir = 'io')),
			Subsignal('tx_clk', Pins('C20', dir = 'o')),
			Subsignal('tx_ctrl', Pins('E19', dir = 'o')),
			Subsignal('tx_data', Pins('J17 J16 D19 D20', dir = 'o')),
			Subsignal('rx_clk', Pins('J19', dir = 'i')),
			Subsignal('rx_ctrl', Pins('F19', dir = 'i')),
			Subsignal('rx_data', Pins('G18 G16 H18 H17', dir = 'i')),
			Attrs(IO_TYPE = 'LVCMOS25')
		),
		# TODO(aki): Replace with `EthernetResource` when it has SGMII support
		Resource(
			'eth_sgmii', 1,
			Subsignal('rst', PinsN('F20', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS25')),
			Subsignal('mdc', Pins('G19', dir = 'o'), Attrs(IO_TYPE = 'LVCMOS25')),
			Subsignal('mdio', Pins('H20', dir = 'io'), Attrs(IO_TYPE = 'LVCMOS25')),
			Subsignal('tx', DiffPairs('W17', 'W18', dir = 'o')),
			Subsignal('rx', DiffPairs('Y16', 'Y17', dir = 'i')),
		),
		# TODO(aki): Replace with `DDR3Resource` when more granular attrs are possible
		Resource(
			'ddr3', 0,
			Subsignal('rst', PinsN('N4', dir = 'o')),
			Subsignal('clk', DiffPairs('M4', 'N5', dir = 'o'), Attrs(IO_TYPE = 'SSTL135D_I')),
			Subsignal('clk_en', Pins('N2', dir = 'o')),
			Subsignal('cs', PinsN('K1', dir = 'o')),
			Subsignal('we', PinsN('M1', dir = 'o')),
			Subsignal('ras', PinsN('P1', dir = 'o')),
			Subsignal('cas', PinsN('L1', dir = 'o')),
			Subsignal('a', Pins('P2 C4 E5 F5 B3 F4 B5 E4 C5 E3 D5 B4 C3', dir = 'o')),
			Subsignal('ba', Pins('P5 N3 M3', dir = 'o')),
			Subsignal(
				'dqs', DiffPairs('K2 H4', 'J1 G5', dir = 'io'),
				Attrs(IO_TYPE = 'SSTL135D_I', DIFFRESISTOR = '100', TERMINATION = 'OFF')
			),
			Subsignal(
				'dq', Pins('L5 F1 K4 G1 L4 H1 G2 J3 D1 C1 E2 C2 F3 A2 E1 B1', dir = 'io'), Attrs(TERMINATION = '75')
			),
			Subsignal('dm', Pins('J4 H5', dir = 'o')),
			Subsignal('odt', Pins('L2', dir = 'o')),
			Attrs(IO_TYPE = 'SSTL135_I', SLEWRATE = 'FAST')
		)
	]
	connectors = [
		Connector(
			'expcon', 1,
			'-   -   -   B19 B12 B9  E6  D6  E7  D7  B11 B6  E9  D9  B8  C8  D8  E8  C7  C6 '
			'-   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -  '
		),  # X3
		Connector(
			'expcon', 2,
			'A8  -   A12 A13 B13 C13 D13 E13 A14 C14 D14 E14 D11 C10 A9  B10 D12 E12 -   - '
			'B15 -   C15 -   D15 -   E15 A16 B16 -   C16 D16 B17 -   C17 A17 B18 A7  A18 - '
		), # X4
	]

	@property
	def file_templates(self) -> dict[str, str]:
		return {
			**super().file_templates,
			'{{name}}-openocd.cfg': dedent(r'''
			interface ftdi
			{# FTDI descriptors is identical between non-5G and 5G recent Versa boards #}
			ftdi_device_desc 'Lattice ECP5_5G VERSA Board'
			ftdi_vid_pid 0x0403 0x6010
			ftdi_channel 0
			ftdi_layout_init 0xfff8 0xfffb
			reset_config none
			adapter_khz 25000

			# ispCLOCK device (unusable with openocd and must be bypassed)
			#jtag newtap ispclock tap -irlen 8 -expected-id 0x00191043
			# ECP5 device
			{% if '5G' in platform.device -%}
			jtag newtap ecp5 tap -irlen 8 -expected-id 0x81112043 ; # LFE5UM5G-45F
			{% else -%}
			jtag newtap ecp5 tap -irlen 8 -expected-id 0x01112043 ; # LFE5UM-45F
			{% endif %}
			''')
		}

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		openocd = environ.get('OPENOCD', 'openocd')
		with products.extract(f'{name}-openocd.cfg', f'{name}.svf') as files:
			(config_filename, vector_filename) = files
			check_call([
				openocd, '-f', config_filename,
				'-c', f'transport select jtag; init; svf -quiet {vector_filename}; exit'
			])


if __name__ == '__main__':
	from ...test.blinky import Blinky
	VersaECP5Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from textwrap                           import dedent

from torii.build                        import Attrs, Clock, Connector, DiffPairs, Pins, PinsN, Resource, Subsignal
from torii.build.run                    import BuildPlan, BuildProducts
from torii.hdl.ir                       import Fragment
from torii.platform.resources.display   import Display7SegResource, VGAResource
from torii.platform.resources.interface import PS2Resource, UARTResource
from torii.platform.resources.memory    import SDCardResources, SPIFlashResources
from torii.platform.resources.user      import LEDResources, RGBLEDResource, SwitchResources
from torii.platform.vendor.xilinx       import XilinxPlatform

__all__ = (
	'Nexys4DDRPlatform',
)

class Nexys4DDRPlatform(XilinxPlatform):
	device      = 'xc7a100t'
	package     = 'csg324'
	speed       = '1'
	default_clk = 'clk100'
	default_rst = 'rst'

	pretty_name = 'Nexys 4 DDR'
	description = 'Digilent Nexys 4 DDR Xilinx Artix7-100T Development Board'

	resources   = [
		Resource('clk100', 0, Pins('E3', dir = 'i'), Clock(100e6), Attrs(IOSTANDARD = 'LVCMOS33')),
		Resource('rst', 0, PinsN('C12', dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),
		*SwitchResources(
			pins = {
				0: 'J15', 1: 'L16', 2: 'M13', 3: 'R15', 4: 'R17',
				5: 'T18', 6: 'U18', 7: 'R13', 10: 'R16', 11: 'T13',
				12: 'H6', 13: 'U12', 14: 'U11', 15: 'V10'
			},
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		*SwitchResources(
			pins = {8: 'T8', 9: 'U8'},
			attrs = Attrs(IOSTANDARD = 'LVCMOS18')
		),
		*LEDResources(
			pins = 'H17 K15 J13 N14 R18 V17 U17 U16 V16 T15 U14 T16 V15 V14 V12 V11',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		RGBLEDResource(
			0,
			r = 'N15', g = 'M16', b = 'R12',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		RGBLEDResource(
			1,
			r = 'N16', g = 'R11', b = 'G14',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Display7SegResource(
			0,
			a = 'T10', b = 'R10', c = 'K16', d = 'K13', e = 'P15',
			f = 'T11', g = 'L18', dp = 'H15', invert = True,
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource(
			'display_7seg_an', 0,
			PinsN('J17 J18 T9 J14 P14 T14 K2 U13', dir = 'o'),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource('button_reset', 0, PinsN('C12', dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),
		Resource('button_center', 0, Pins('N17', dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),
		Resource('button_up', 0, Pins('M18', dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),
		Resource('button_left', 0, Pins('P17', dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),
		Resource('button_right', 0, Pins('M17', dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),
		Resource('button_down', 0, Pins('P18', dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),
		VGAResource(
			0,
			r = 'A3 B4 C5 A4',
			g = 'C6 A5 B6 A6',
			b = 'B7 C7 D7 D8',
			hs = 'B11', vs = 'B12',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		*SDCardResources(
			0,
			clk = 'B1',  cmd = 'C1',  cd = 'A1',
			dat0 = 'C2', dat1 = 'E1', dat2 = 'F1', dat3 = 'D2',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource('sd_card_reset', 0, Pins('E2', dir = 'o'), Attrs(IOSTANDARD = 'LVCMOS33')),
		Resource(
			'accelerometer', 0,                        # ADXL362
			Subsignal('cs', PinsN('D15', dir = 'o')),
			Subsignal('clk', Pins('F15', dir = 'o')),
			Subsignal('copi', Pins('F14', dir = 'o')),
			Subsignal('cipo', Pins('E15', dir = 'i')),
			Subsignal('int', Pins('B13 C16', dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33', PULLUP = 'TRUE')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource(
			'temp_sensor', 0, # ADT7420
			Subsignal('scl', Pins('C14', dir = 'o')),
			Subsignal('sda', Pins('C15', dir = 'io')),
			Subsignal('int', Pins('D13', dir = 'i'), Attrs(PULLUP = 'TRUE')),
			Subsignal('ct', Pins('B14', dir = 'i'), Attrs(PULLUP = 'TRUE')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),

		Resource(
			'microphone', 0, # ADMP421
			Subsignal('clk', Pins('J5', dir = 'o')),
			Subsignal('data', Pins('H5', dir = 'i')),
			Subsignal('lr_sel', Pins('F5', dir = 'o')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource(
			'audio', 0,
			Subsignal('pwm', Pins('A11', dir = 'o')),
			Subsignal('sd', PinsN('D12', dir = 'o')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
		UARTResource(
			0,
			rx = 'C4', tx = 'D4', rts = 'E5', cts = 'D3',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33'),
			role = 'dce'
		),
		PS2Resource(
			0,
			clk = 'F4', dat = 'B2',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33', PULLUP = 'TRUE')
		),
		# TODO(aki): Replace with `EthernetResource` when more granular attrs are possible
		Resource(
			'eth', 0,  # LAN8720A
			Subsignal('mdio', Pins('A9', dir = 'io')),
			Subsignal('mdc', Pins('C9', dir = 'o')),
			Subsignal('reset', Pins('B3', dir = 'o')),
			Subsignal('rxd', Pins('C11 D10', dir = 'io')),
			Subsignal('rxerr', Pins('C10', dir = 'io')),
			Subsignal('txd', Pins('A10 A8', dir = 'o')),
			Subsignal('txen', Pins('B9', dir = 'o')),
			Subsignal('crs_dv', Pins('D9', dir = 'io')),
			Subsignal('int', PinsN('B8', dir = 'io')),
			Subsignal('clk', Pins('D5', dir = 'o'), Clock(50e6)),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
		*SPIFlashResources(
			0,
			cs_n = 'L13', clk = 'E9', copi = 'K17', cipo = 'K18', wp_n = 'L14', hold_n = 'M14',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		# TODO(aki): Replace with `DDR2Resource` when more granular attrs are possible
		Resource(
			'ddr2', 0, # MT47H64M16HR-25:H
			Subsignal('a', Pins('M4 P4 M6 T1 L3 P5 M2 N1 L4 N5 R2 K5 N6 K3', dir = 'o')),
			Subsignal(
				'dq',
				Pins('R7 V6 R8 U7 V7 R6 U6 R5 T5 U3 V5 U4 V4 T4 V1 T3', dir = 'io'),
				Attrs(IN_TERM = 'UNTUNED_SPLIT_50')
			),
			Subsignal('ba', Pins('P2 P3 R1', dir = 'o')),
			Subsignal('clk', DiffPairs('L6', 'L5', dir = 'o'), Attrs(IOSTANDARD = 'DIFF_SSTL18_I')),
			Subsignal('clk_en', Pins('M1', dir = 'o')),
			Subsignal('cs', PinsN('K6', dir = 'o')),
			Subsignal('we', PinsN('N2', dir = 'o')),
			Subsignal('ras', PinsN('N4', dir = 'o')),
			Subsignal('cas', PinsN('L1', dir = 'o')),
			Subsignal('dqs', DiffPairs('U9 U2', 'V9 V2', dir = 'o'), Attrs(IOSTANDARD = 'DIFF_SSTL18_I')),
			Subsignal('dm', Pins('T6 U1', dir = 'o')),
			Subsignal('odt', Pins('R5', dir = 'o')),
			Attrs(IOSTANDARD = 'SSTL18_I', SLEW = 'FAST'))
	]

	connectors = [
		Connector('pmod', 0, 'C17 D18 E18 G17 - - D17 E17 F18 G18 - -'),  # JA
		Connector('pmod', 1, 'D14 F16 G16 H14 - - E16 F13 G13 H16 - -'),  # JB
		Connector('pmod', 2, 'K1 F6 J2 G6 - - E7 J3 J4 E6 - -'),  # JC
		Connector('pmod', 3, 'H4 H1 G1 G3 - - H2 G4 G2 F3 - -')   # JD
	]

	def toolchain_prepare(self, fragment: Fragment, name: str, **kwargs) -> BuildPlan:
		overrides = {
			'script_before_bitstream': 'set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]',
			'script_after_bitstream': dedent(f'''
				write_cfgmem -force -format bin -interface spix4 -size 16 -loadbit 'up 0x0 {name}.bit' -file {name}.bin
			'''),
			'add_constraints': dedent('''
				set_property INTERNAL_VREF 0.9 [get_iobanks 34]
				set_property CFGBVS VCCO [current_design]
				set_property CONFIG_VOLTAGE 3.3 [current_design]
			''')
		}
		return super().toolchain_prepare(fragment, name, **overrides, **kwargs)

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import run

		xc3sprog = environ.get('XC3SPROG', 'xc3sprog')
		with products.extract(f'{name}.bit') as bitstream_filename:
			run([xc3sprog, '-c', 'nexys4', bitstream_filename], check = True)


if __name__ == '__main__':
	from ...test.blinky import Blinky
	Nexys4DDRPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from textwrap                           import dedent

from torii.build                        import Attrs, Clock, Connector, DiffPairs, Pins, PinsN, Resource, Subsignal
from torii.build.run                    import BuildPlan, BuildProducts
from torii.hdl.ir                       import Fragment
from torii.platform.resources.interface import I2CResource, SPIResource, UARTResource
from torii.platform.resources.memory    import SPIFlashResources
from torii.platform.resources.user      import ButtonResources, LEDResources, RGBLEDResource, SwitchResources
from torii.platform.vendor.xilinx       import XilinxPlatform

__all__ = (
	'ArtyS7_25Platform',
	'ArtyS7_50Platform',
)

class _ArtyS7Platform(XilinxPlatform):
	package     = 'csga324'
	speed       = '1'
	default_clk = 'clk100'
	default_rst = 'rst'

	resources   = [
		Resource(
			'clk100', 0, Pins('R2', dir = 'i'), Clock(100e6), Attrs(IOSTANDARD = 'SSTL135')
		),
		Resource('rst', 0, PinsN('C18', dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),
		*LEDResources(pins = 'E18 F13 E13 H15', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		RGBLEDResource(0, r = 'J15', g = 'G17', b = 'F15', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		RGBLEDResource(1, r = 'E15', g = 'F18', b = 'E14', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		*ButtonResources(pins = 'G15 K16 J16 H13', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		*SwitchResources(pins = 'H14 H18 G18  M5', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		UARTResource(
			0,
			rx = 'V12', tx = 'R12',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		SPIResource(
			0,
			cs_n = 'H16', clk = 'G16', copi = 'H17', cipo = 'K14',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		I2CResource(
			0,
			scl = 'J14', sda = 'J13',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		*SPIFlashResources(
			0,
			cs_n = 'M13', clk = 'D11', copi = 'K17', cipo = 'K18', wp_n = 'L14', hold_n = 'M15',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		# TODO(aki): Replace with `DDR3Resource` when more granular attrs are possible
		Resource(
			'ddr3', 0,
			Subsignal('rst', PinsN('J6', dir = 'o')),
			Subsignal('clk', DiffPairs('R5', 'T4', dir = 'o'), Attrs(IOSTANDARD = 'DIFF_SSTL135')),
			Subsignal('clk_en', Pins('T2', dir = 'o')),
			Subsignal('cs', PinsN('R3', dir = 'o')),
			Subsignal('we', PinsN('P7', dir = 'o')),
			Subsignal('ras', PinsN('U1', dir = 'o')),
			Subsignal('cas', PinsN('V3', dir = 'o')),
			Subsignal('a', Pins('U2 R4 V2 V4 T3 R7 V6 T6 U7 V7 P6 T5 R6 U6', dir = 'o')),
			Subsignal('ba', Pins('V5 T1 U3', dir = 'o')),
			Subsignal(
				'dqs', DiffPairs('K1 N3', 'L1 N2', dir = 'io'), Attrs(IOSTANDARD = 'DIFF_SSTL135')
			),
			Subsignal(
				'dq', Pins('K2 K3 L4 M6 K6 M4 L5 L6 N4 R1 N1 N5 M2 P1 M1 P2', dir = 'io'),
				Attrs(IN_TERM = 'UNTUNED_SPLIT_40')
			),
			Subsignal('dm', Pins('K4 M3', dir = 'o')),
			Subsignal('odt', Pins('P5', dir = 'o')),
			Attrs(IOSTANDARD = 'SSTL135', SLEW = 'FAST'),
		),
	]
	connectors  = [
		Connector('pmod', 0, 'L17 L18 M14 N14 - - M16 M17 M18 N18 - -'), # JA
		Connector('pmod', 1, 'P17 P18 R18 T18 - - P14 P15 N15 P16 - -'), # JB
		Connector('pmod', 2, 'U15 V16 U17 U18 - - U16 P13 R13 V14 - -'), # JC
		Connector('pmod', 3, 'V15 U12 V13 T12 - - T13 R11 T11 U11 - -'), # JD
		Connector('ck_io', 0, {
			# Outer Digital Header
			'io0': 'L13',
			'io1': 'N13',
			'io2': 'L16',
			'io3': 'R14',
			'io4': 'T14',
			'io5': 'R16',
			'io6': 'R17',
			'io7': 'V17',
			'io8': 'R15',
			'io9': 'T15',
			'io10': 'H16',
			'io11': 'H17',
			'io12': 'K14',
			'io13': 'G16',
			# Inner Digital Header
			'io26': 'U11',
			'io27': 'T11',
			'io28': 'R11',
			'io29': 'T13',
			'io30': 'T12',
			'io31': 'V13',
			'io32': 'U12',
			'io33': 'V15',
			'io34': 'V14',
			'io35': 'R13',
			'io36': 'P13',
			'io37': 'U16',
			'io38': 'U18',
			'io39': 'U17',
			'io40': 'V16',
			'io41': 'U15',
			# Outer Analog Header as Digital IO
			'a0': 'G13',
			'a1': 'B16',
			'a2': 'A16',
			'a3': 'C13',
			'a4': 'C14',
			'a5': 'D18',
			# Inner Analog Header as Digital IO
			'io20': 'B14',
			'io21': 'A14',
			'io22': 'D16',
			'io23': 'D17',
			'io24': 'D14',
			'io25': 'D15'
		}),
		Connector('xadc', 0, {
			# Outer Analog Header
			'vaux0_p': 'B13',
			'vaux0_n': 'A13',
			'vaux1_p': 'B15',
			'vaux1_n': 'A15',
			'vaux9_p': 'E12',
			'vaux9_n': 'D12',
			'vaux2_p': 'B17',
			'vaux2_n': 'A17',
			'vaux10_p': 'C17',
			'vaux10_n': 'B18',
			'vaux11_p': 'E16',
			'vaux11_n': 'E17',
			# Inner Analog Header
			'vaux8_p': 'B14',
			'vaux8_n': 'A14',
			'vaux3_p': 'D16',
			'vaux3_n': 'D17',
		})
	]

	def toolchain_prepare(self, fragment: Fragment, name: str, **kwargs) -> BuildPlan:
		overrides = {
			'script_before_bitstream': 'set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]',
			'script_after_bitstream': dedent(f'''
				write_cfgmem -force -format mcs -interface spix4 -size 16 -loadbit 'up 0x0 {name}.bit' -file {name}.mcs
			'''),
			'add_constraints': 'set_property INTERNAL_VREF 0.675 [get_iobanks 34]'
		}
		return super().toolchain_prepare(fragment, name, **overrides, **kwargs)

	def _program_vivado(self, products: BuildProducts, name: str, flash: bool) -> None:
		from subprocess  import run
		from torii.tools import require_tool

		vivado = require_tool('vivado')

		# It does not appear possible to reset the FPGA via TCL after
		# flash programming.
		with products.extract(f'{name}.bin') as bitstream:
			cmd_script = dedent('''
				open_hw_manager
				connect_hw_server
				open_hw_target
				current_hw_device [lindex [get_hw_devices] 0]
			''')

			if flash:
				cmd_script += dedent(f'''
					create_hw_cfgmem -hw_device [current_hw_device] s25fl128sxxxxxx0-spi-x1_x2_x4
					set_property PROGRAM.FILES {{{bitstream}}} [current_hw_cfgmem]
					set_property PROGRAM.ADDRESS_RANGE  {{use_file}} [current_hw_cfgmem]
					set_property PROGRAM.BLANK_CHECK  1 [current_hw_cfgmem]
					set_property PROGRAM.ERASE  1 [current_hw_cfgmem]
					set_property PROGRAM.CFG_PROGRAM  1 [current_hw_cfgmem]
					set_property PROGRAM.VERIFY  1 [current_hw_cfgmem]
					create_hw_bitstream -hw_device [current_hw_device] [get_property PROGRAM.HW_CFGMEM_BITFILE [current_hw_device]]
					program_hw_devices
					program_hw_cfgmem
				''') # noqa: E501
			else:
				cmd_script += dedent(f'''
					set_property PROGRAM.FILE {{{bitstream}}} [current_hw_device]
					program_hw_devices
				''')

			cmd_script += dedent('''
				close_hw_manager
				puts "Reset or power-cycle your board now."
			''')

			run([vivado, '-nolog', '-nojournal', '-mode', 'tcl'], input = cmd_script.encode('utf-8'), check = True)

	def _program_openocd(self, products: BuildProducts, name: str, flash: bool) -> None:
		from os         import environ
		from subprocess import check_call

		openocd = environ.get('OPENOCD', 'openocd')
		# In order, OpenOCD searches these directories for files:
		# * $HOME/.openocd if $HOME exists (*nix)
		# * Path pointed to by $OPENOCD_SCRIPTS if $OPENOCD_SCRIPTS exists
		# * $APPDATA/OpenOCD on Windows
		# Place the bscan_spi_xc7s50.bit proxy bitstream under a directory
		# named 'proxy' in one of the above directories so OpenOCD finds it.

		with products.extract(f'{name}.bit') as bitstream:
			command_str = 'init;'
			if flash:
				command_str += 'jtagspi_init 0 [find proxy/bscan_spi_xc7s50.bit];'
				command_str += f'jtagspi_program {bitstream} 0;'
				command_str += 'xc7_program xc7.tap;'
			else:
				command_str += f'pld load 0 {bitstream};'
			command_str += 'shutdown'

			check_call([openocd, '-f', 'board/arty_s7.cfg', '-c', command_str])

	def toolchain_program(
		self, products: BuildProducts, name: str, *, programmer: str = 'vivado', flash: bool = True
	) -> None:

		if programmer not in ('vivado', 'openocd'):
			raise ValueError(f'programmer must be either \'vivado\' or \'openocd\', not {programmer!r}')

		if programmer == 'vivado':
			self._program_vivado(products, name, flash)
		else:
			self._program_openocd(products, name, flash)


class ArtyS7_50Platform(_ArtyS7Platform):
	device      = 'xc7s50'

	pretty_name = 'Arty S7-50'
	description = 'Digilent Arty S7-50 Xilinx Spartan7-50 Development Board'

class ArtyS7_25Platform(_ArtyS7Platform):
	device      = 'xc7s25'

	pretty_name = 'Arty S7-25'
	description = 'Digilent Arty S7-25 Xilinx Spartan7-25 Development Board'

if __name__ == '__main__':
	from ...test.blinky import Blinky
	ArtyS7_25Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                         import Attrs, Clock, Connector, Pins, PinsN, Resource
from torii.build.run                     import BuildProducts
from torii.platform.resources.interface  import UARTResource
from torii.platform.resources.memory     import SPIFlashResources
from torii.platform.resources.user       import ButtonResources, LEDResources
from torii.platform.vendor.lattice.ice40 import ICE40Platform

__all__ = (
	'ICEBreakerPlatform',
)

class ICEBreakerPlatform(ICE40Platform):
	device      = 'iCE40UP5K'
	package     = 'SG48'
	default_clk = 'clk12'

	pretty_name = 'iCEBreaker'
	description = '1BitSquared iCEBreaker Lattice iCE40-UP5K Development Board'

	resources   = [
		Resource(
			'clk12', 0, Pins('35', dir = 'i'), Clock(12e6),
			Attrs(GLOBAL = True, IO_STANDARD = 'SB_LVCMOS')
		),
		*LEDResources(pins = '11 37', invert = True, attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),
		# Semantic aliases
		Resource('led_r', 0, PinsN('11', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('led_g', 0, PinsN('37', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		*ButtonResources(pins = '10', invert = True, attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),
		UARTResource(
			0,
			rx = '6', tx = '9',
			attrs = Attrs(IO_STANDARD = 'SB_LVTTL', PULLUP = 1)
		),
		*SPIFlashResources(
			0,
			cs_n = '16', clk = '15', copi = '14', cipo = '17', wp_n = '12', hold_n = '13',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
	]
	connectors = [
		Connector('pmod', 0, '4 2 47 45 - -  3 48 46 44 - -'), # PMOD1A
		Connector('pmod', 1, '43 38 34 31 - - 42 36 32 28 - -'), # PMOD1B
		Connector('pmod', 2, '27 25 21 19 - - 26 23 20 18 - -'), # PMOD2
	]
	# The attached LED/button section can be either used standalone or as a PMOD.
	# Attach to platform using:
	# p.add_resources(p.break_off_pmod)
	# pmod_btn = plat.request('user_btn')
	break_off_pmod = [
		*LEDResources(
			pins = {
				2: '7', 3: '1', 4: '2', 5: '8', 6: '3'
			}, conn = ('pmod', 2), attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
		# Semantic aliases
		Resource(
			'led_r', 1, Pins('7', dir = 'o', conn = ('pmod', 2)), Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
		Resource(
			'led_g', 1, Pins('1', dir = 'o', conn = ('pmod', 2)), Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
		Resource(
			'led_g', 2, Pins('2', dir = 'o', conn = ('pmod', 2)), Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
		Resource(
			'led_g', 3, Pins('8', dir = 'o', conn = ('pmod', 2)), Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
		Resource(
			'led_g', 4, Pins('3', dir = 'o', conn = ('pmod', 2)), Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
		*ButtonResources(
			pins = {
				1: '9', 2: '4', 3: '10'
			}, conn = ('pmod', 2), attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		iceprog = environ.get('ICEPROG', 'iceprog')
		with products.extract(f'{name}.bin') as bitstream_filename:
			check_call([iceprog, bitstream_filename])


if __name__ == '__main__':
	from ...test.blinky import Blinky
	p = ICEBreakerPlatform()
	p.add_resources(p.break_off_pmod)
	p.build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                        import Attrs, Clock, Connector, Pins, Resource
from torii.build.run                    import BuildProducts
from torii.platform.resources.display   import Display7SegResource, VGAResource
from torii.platform.resources.interface import UARTResource
from torii.platform.resources.memory    import SDRAMResource
from torii.platform.resources.user      import ButtonResources, LEDResources, SwitchResources
from torii.platform.vendor.altera       import AlteraPlatform

__all__ = (
	'DE10LitePlatform',
)

class DE10LitePlatform(AlteraPlatform):
	device      = '10M50DA' # MAX10
	package     = 'F484'    # FBGA-484
	speed       = 'C7'
	suffix      = 'G'
	default_clk = 'clk50'

	pretty_name = 'DE10-Lite'
	description = 'terasIC DE10-Lite Altera MAX 10 Development Board'

	resources   = [
		Resource(
			'clk10', 0, Pins('N5', dir = 'i'),  Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'clk50', 0, Pins('P11', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		Resource(
			'clk50', 1, Pins('N14', dir = 'i'), Clock(50e6), Attrs(io_standard = '3.3-V LVTTL')
		),
		*LEDResources(
			pins = 'A8 A9 A10 B10 D13 C13 E14 D14 A11 B11',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		*ButtonResources(
			pins = 'B8 A7', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		*SwitchResources(
			pins = 'C10 C11 D12 C12 A12 B12 A13 A14 B14 F15',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(
			0,
			a = 'C14', b = 'E15', c = 'C15', d = 'C16', e = 'E16', f = 'D17', g = 'C17', dp = 'D15', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(
			1,
			a = 'C18', b = 'D18', c = 'E18', d = 'B16', e = 'A17', f = 'A18', g = 'B17', dp = 'A16', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(
			2,
			a = 'B20', b = 'A20', c = 'B19', d = 'A21', e = 'B21', f = 'C22', g = 'B22', dp = 'A19', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(
			3,
			a = 'F21', b = 'E22', c = 'E21', d = 'C19', e = 'C20', f = 'D19', g = 'E17', dp = 'D22', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(
			4,
			a = 'F18', b = 'E20', c = 'E19', d = 'J18', e = 'H19', f = 'F19', g = 'F20', dp = 'F17', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		Display7SegResource(
			5,
			a = 'J20', b = 'K20', c = 'L18', d = 'N18', e = 'M20', f = 'N19', g = 'N20', dp = 'L19', invert = True,
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		# Arduino header
		UARTResource(
			0,
			rx = 'V10', tx = 'W10',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		),
		SDRAMResource(
			0,
			clk = 'L14', cs_n = 'U20', we_n = 'V20', ras_n = 'U22', cas_n = 'U21',
			ba = 'T21 T22', a = 'U17 W19 V18 U18 U19 T18 T19 R18 P18 P19 T20 P20 R20',
			dq = 'Y21 Y20 AA22 AA21 Y22 W22 W20 V21 P21 J22 H21 H22 G22 G20 G19 F22',
			dqm = 'V22 J21', attrs = Attrs(io_standard = '3.3-V LVCMOS')
		),
		VGAResource(
			0,
			r = 'AA1 V1 Y2 Y1',
			g = 'W1 T2 R2 R1',
			b = 'P1 T1 P4 N2',
			hs = 'N3', vs = 'N1',
			attrs = Attrs(io_standard = '3.3-V LVTTL')
		)
	]

	connectors  = [
		Connector(
			'gpio', 0,
			'V10  W10 V9 W9  V8 W8  V7 W7  W6 V5  W5 AA15 AA14 W13 W12 AB13 AB12 Y11 AB11 W11 AB10 '
			'AA10 AA9 Y8 AA8 Y7 AA7 Y6 AA6 Y5 AA5 Y4 AB3  Y3   AB2 AA2 -    -    -   -    -   -    '
		),
		Connector(
			'gpio', 5,
			'AB5 AB6 AB7 AB8 AB9 Y10 AA11 AA12 AB17 AA17 AB19 AA19 Y19 AB20 AB21 AA20 F16'
		),
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import check_call

		quartus_pgm = environ.get('QUARTUS_PGM', 'quartus_pgm')
		with products.extract(f'{name}.sof') as bitstream_filename:
			check_call([
				quartus_pgm, '--haltcc', '--mode', 'JTAG',
				'--operation', 'P;' + bitstream_filename
			])


if __name__ == '__main__':
	from ...test.blinky import Blinky
	DE10LitePlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                         import Attrs, Clock, Connector, Pins, PinsN, Resource
from torii.build.run                     import BuildProducts
from torii.platform.resources.interface  import DirectUSBResource
from torii.platform.resources.memory     import SPIFlashResources
from torii.platform.resources.user       import ButtonResources, LEDResources, RGBLEDResource
from torii.platform.vendor.lattice.ice40 import ICE40Platform

__all__ = (
	'ICEBreakerBitsyPlatform',
)

class ICEBreakerBitsyPlatform(ICE40Platform):
	device      = 'iCE40UP5K'
	package     = 'SG48'
	default_clk = 'clk12'

	pretty_name = 'iCEBreaker Bitsy'
	description = '1BitSquared iCEBreaker Bitsy Lattice iCE40-UP5K Development Board'

	resources   = [
		Resource(
			'clk12', 0, Pins('35', dir = 'i'), Clock(12e6),
			Attrs(GLOBAL = True, IO_STANDARD = 'SB_LVCMOS')
		),
		DirectUSBResource(
			0,
			d_p = '42', d_n = '38', pullup = '37',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
		*SPIFlashResources(
			0,
			cs_n = '16', clk = '15', copi = '14', cipo = '17', wp_n = '18', hold_n = '19',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
		RGBLEDResource(
			0,
			r = '39', g = '40', b = '41', invert = True,
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')
		),
		*LEDResources(pins = '25 6', invert = True, attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('led_r', 0, PinsN('25', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('led_g', 0, PinsN('6', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		*ButtonResources(pins = '2', invert = True, attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),
	]
	connectors = [
		Connector('edge', 0, { # Pins bottom P0 - P12,
				'0': '47', '1': '44', '2': '48', '3': '45', '4': '4', '5': '3',
				'6': '9', '7': '10', '8': '11', '9': '12', '10': '21', '11': '13',
				'12': '20', '13': '25', '14': '23', '15': '27', '16': '26', '17': '28',
				'18': '31', '19': '32', '20': '34', '21': '36', '22': '43', '23': '46'
			}
		),
		Connector('pmod', 1, '0 2 4 6 - - 1 3 5 7 - -', conn = ('edge', '0')), # PMOD 1
		Connector('pmod', 2, '22 19 16 17 - - 21 18 15 20 - -', conn = ('edge', '0')), # PMOD 2
		Connector('pmod', 3, '14 9 11 8 - - 13 10 12 23 - -', conn = ('edge', '0'))  # PMOD 3
	]

	def toolchain_program(
		self, products: BuildProducts, name: str,
		run_vid: str | None = None, run_pid: str | None = None,
		dfu_vid: str = '1d50', dfu_pid: str = '6146', reset: bool = True
	) -> None:
		from os         import environ
		from subprocess import check_call

		dfu_util = environ.get('DFU_UTIL', 'dfu-util')

		# Construct the device runtime and DFU vid pid string
		dev_str = ''
		if run_vid or run_pid:
			dev_str = f'{run_vid or ""}:{run_pid or ""}'
		dev_str += f',{dfu_vid or ""}:{dfu_pid or ""}'

		# Construct the argument list for dfu-util
		args = [dfu_util, '-d', dev_str, '-a', '0']
		if reset:
			args.append('-R')
		args.append('-D')

		# Run dfu-util
		with products.extract(f'{name}.bin') as bitstream_filename:
			args.append(bitstream_filename)
			check_call(args)


if __name__ == '__main__':
	from ...test.blinky import Blinky
	ICEBreakerBitsyPlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from .._util import _deprecated_import

__all__ = (
	'arrow_deca',
	'chameleon96',
	'de0_cv',
	'de0',
	'de1_soc',
	'de10_lite',
	'de10_nano',
	'mister',
	'rz_easyfpga_a2_2',
)

def __dir__() -> list[str]:
	return list({*globals(), *__all__})

def __getattr__(name: str):
	match name:
		case 'arrow_deca':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.max10.{name}')
			from .max10 import arrow_deca
			return arrow_deca
		case 'chameleon96':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.cyclone_v.{name}')
			from .cyclone_v import chameleon96
			return chameleon96
		case 'de0_cv':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.cyclone_v.{name}')
			from .cyclone_v import de0_cv
			return de0_cv
		case 'de0':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.cyclone_iii.{name}')
			from .cyclone_iii import de0
			return de0
		case 'de1_soc':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.cyclone_v.{name}')
			from .cyclone_v import de1_soc
			return de1_soc
		case 'de10_lite':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.max10.{name}')
			from .max10 import de10_lite
			return de10_lite
		case 'de10_nano':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.cyclone_v.{name}')
			from .cyclone_v import de10_nano
			return de10_nano
		case 'mister':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.cyclone_v.{name}')
			from .cyclone_v import mister
			return mister
		case 'rz_easyfpga_a2_2':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.cyclone_iv.{name}')
			from .cyclone_iv import rz_easyfpga_a2_2
			return rz_easyfpga_a2_2
		case _:
			raise AttributeError(f'module {__name__!r} has no attribute {name!r}')



# SPDX-License-Identifier: BSD-2-Clause

from .._util import _deprecated_import

__all__ = (
	'alchitry_au',
	'arty_a7',
	'arty_s7',
	'arty_z7',
	'atlys',
	'cmod_a7',
	'cmod_s7',
	'ebaz4205',
	'genesys2',
	'kc705',
	'kcu105',
	'mega65',
	'mercury',
	'microzed_z010',
	'microzed_z020',
	'nexys4ddr',
	'numato_mimas',
	'sk_xc6slx9',
	'te0714_03_50_2I',
	'zturn_lite_z007s',
	'zturn_lite_z010',
)

def __dir__() -> list[str]:
	return list({*globals(), *__all__})

def __getattr__(name: str):
	match name:
		case 'alchitry_au':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.artix7.{name}')
			from .artix7 import alchitry_au
			return alchitry_au
		case 'arty_a7':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.artix7.{name}')
			from .artix7 import arty_a7
			return arty_a7
		case 'arty_s7':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.spartan7.{name}')
			from .spartan7 import arty_s7
			return arty_s7
		case 'arty_z7':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.zynq7.{name}')
			from .zynq7 import arty_z7
			return arty_z7
		case 'atlys':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.spartan6.{name}')
			from .spartan6 import atlys
			return atlys
		case 'cmod_a7':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.artix7.{name}')
			from .artix7 import cmod_a7
			return cmod_a7
		case 'cmod_s7':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.spartan7.{name}')
			from .spartan7 import cmod_s7
			return cmod_s7
		case 'ebaz4205':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.zynq7.{name}')
			from .zynq7 import ebaz4205
			return ebaz4205
		case 'genesys2':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.kintex7.{name}')
			from .kintex7 import genesys2
			return genesys2
		case 'kc705':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.kintex7.{name}')
			from .kintex7 import kc705
			return kc705
		case 'kcu105':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.kintex_us.{name}')
			from .kintex_us import kcu105
			return kcu105
		case 'mega65':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.artix7.{name}')
			from .artix7 import mega65
			return mega65
		case 'mercury':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.spartan3.{name}')
			from .spartan3 import mercury
			return mercury
		case 'microzed_z010':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.zynq7.{name}')
			from .zynq7 import microzed_z010
			return microzed_z010
		case 'microzed_z020':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.zynq7.{name}')
			from .zynq7 import microzed_z020
			return microzed_z020
		case 'nexys4ddr':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.artix7.{name}')
			from .artix7 import nexys4ddr
			return nexys4ddr
		case 'numato_mimas':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.spartan6.{name}')
			from .spartan6 import numato_mimas
			return numato_mimas
		case 'sk_xc6slx9':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.spartan6.{name}')
			from .spartan6 import sk_xc6slx9
			return sk_xc6slx9
		case 'te0714_03_50_2I':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.artix7.{name}')
			from .artix7 import te0714_03_50_2I
			return te0714_03_50_2I
		case 'zturn_lite_z007s':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.zynq7.{name}')
			from .zynq7 import zturn_lite_z007s
			return zturn_lite_z007s
		case 'zturn_lite_z010':
			_deprecated_import(f'{__name__}.{name}', f'{__name__}.zynq7.{name}')
			from .zynq7 import zturn_lite_z010
			return zturn_lite_z010
		case _:
			raise AttributeError(f'module {__name__!r} has no attribute {name!r}')



# SPDX-License-Identifier: BSD-2-Clause

from torii.build                         import Attrs, Clock, Connector, Pins, Resource
from torii.build.run                     import BuildProducts
from torii.platform.resources.interface  import UARTResource
from torii.platform.resources.memory     import SRAMResource
from torii.platform.resources.user       import ButtonResources, LEDResources, SwitchResources
from torii.platform.vendor.lattice.ice40 import ICE40Platform

__all__ = (
	'BlackIcePlatform',
)

class BlackIcePlatform(ICE40Platform):
	device      = 'iCE40HX4K'
	package     = 'TQ144'
	default_clk = 'clk100'

	pretty_name = 'BlackIce'
	description = 'myStorm BlackIce Lattice iCE40-HX4k Development Board'

	resources   = [
		Resource(
			'clk100', 0, Pins('129', dir = 'i'), Clock(100e6), Attrs(GLOBAL = True, IO_STANDARD = 'SB_LVCMOS')
		),
		*LEDResources(pins = '71 67 68 70', attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),
		# Semantic aliases
		Resource('led_b', 0, Pins('71', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('led_g', 0, Pins('67', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('led_o', 0, Pins('68', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		Resource('led_r', 0, Pins('70', dir = 'o'), Attrs(IO_STANDARD = 'SB_LVCMOS')),
		*ButtonResources(pins = '63 64', invert = True, attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),
		*SwitchResources(pins = '37 38 39 41', invert = True, attrs = Attrs(IO_STANDARD = 'SB_LVCMOS')),
		UARTResource(
			0,
			rx = '88', tx = '85',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS', PULLUP = 1),
			role = 'dce'
		),
		SRAMResource(
			0,
			cs_n = '136', oe_n = '45', we_n = '120',
			a = '137 138 139 141 142 42 43 44 73 74 75 76 115 116 117 118 119 78',
			d = '135 134 130 128 125 124 122 121 61 60 56 55 52 49 48 47',
			attrs = Attrs(IO_STANDARD = 'SB_LVCMOS'),
		),
	]
	connectors  = [
		Connector('pmod', 0, '94 91 88 85 - - 95 93 90 87 - -'),  # PMOD1/2
		Connector('pmod', 1, '105 102 99 97 - - 104 101 98 96 - -'),  # PMOD3/4
		Connector('pmod', 2, '143 114 112 107 - - 144 113 110 106 - -'),  # PMOD5/6
		Connector('pmod', 3, '10 9 2 1 - - 8 7 4 3 - -'),  # PMOD7/8
		Connector('pmod', 4, '20 19 16 15 - - 18 17 12 11 - -'),  # PMOD9/10
		Connector('pmod', 5, '34 33 22 21 - - 32 31 26 25 - -'),  # PMOD11/12
		Connector('pmod', 6, '29 28 24 23 - -'),  # PMOD13
		Connector('pmod', 7, '71 67 68 70 - -'),  # PMOD14
	]

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from subprocess import check_call

		with products.extract(f'{name}.bin') as bitstream_filename:
			check_call([
				'cp', bitstream_filename, '/dev/ttyACM0'
			])


if __name__ == '__main__':
	from ...test.blinky import Blinky
	BlackIcePlatform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from textwrap                           import dedent

from torii.build                        import Attrs, Clock, Connector, DiffPairs, Pins, PinsN, Resource, Subsignal
from torii.build.run                    import BuildPlan, BuildProducts
from torii.hdl.ir                       import Fragment
from torii.platform.resources.interface import SPIResource, UARTResource
from torii.platform.resources.memory    import SPIFlashResources
from torii.platform.resources.user      import ButtonResources, LEDResources, RGBLEDResource, SwitchResources
from torii.platform.vendor.xilinx       import XilinxPlatform

__all__ = (
	'ArtyA7_35Platform',
	'ArtyA7_100Platform',
)

class _ArtyA7Platform(XilinxPlatform):
	package     = 'csg324'
	speed       = '1L'
	default_clk = 'clk100'
	default_rst = 'rst'

	resources   = [
		Resource(
			'clk100', 0, Pins('E3', dir = 'i'), Clock(100e6), Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource('rst', 0, PinsN('C2', dir = 'i'), Attrs(IOSTANDARD = 'LVCMOS33')),
		*LEDResources(pins = 'H5 J5 T9 T10', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		RGBLEDResource(0, r = 'G6', g = 'F6', b = 'E1', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		RGBLEDResource(1, r = 'G3', g = 'J4', b = 'G4', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		RGBLEDResource(2, r = 'J3', g = 'J2', b = 'H4', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		RGBLEDResource(3, r = 'K1', g = 'H6', b = 'K2', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		*ButtonResources(pins = 'D9  C9  B9  B8 ', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		*SwitchResources(pins = 'A8  C11 C10 A10', attrs = Attrs(IOSTANDARD = 'LVCMOS33')),
		UARTResource(
			0,
			rx = 'A9', tx = 'D10',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		SPIResource(
			0,
			cs_n = 'C1', clk = 'F1', copi = 'H1', cipo = 'G1',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource(
			'i2c', 0,
			Subsignal('scl',  Pins('L18', dir = 'io')),
			Subsignal('sda', Pins('M18', dir = 'io')),
			Subsignal('scl_pullup', Pins('A14', dir = 'o')),
			Subsignal('sda_pullup', Pins('A13', dir = 'o')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
		*SPIFlashResources(
			0,
			cs_n = 'L13', clk = 'L16', copi = 'K17', cipo = 'K18', wp_n = 'L14', hold_n = 'M14',
			attrs = Attrs(IOSTANDARD = 'LVCMOS33')
		),
		# TODO(aki): Replace with `DDR3Resource` when more granular attrs are possible
		Resource(
			'ddr3', 0,
			Subsignal('rst', PinsN('K6', dir = 'o')),
			Subsignal('clk', DiffPairs('U9', 'V9', dir = 'o'), Attrs(IOSTANDARD = 'DIFF_SSTL135')),
			Subsignal('clk_en', Pins('N5', dir = 'o')),
			Subsignal('cs', PinsN('U8', dir = 'o')),
			Subsignal('we', PinsN('P5', dir = 'o')),
			Subsignal('ras', PinsN('P3', dir = 'o')),
			Subsignal('cas', PinsN('M4', dir = 'o')),
			Subsignal('a', Pins('R2 M6 N4 T1 N6 R7 V6 U7 R8 V7 R6 U6 T6 T8', dir = 'o')),
			Subsignal('ba', Pins('R1 P4 P2', dir = 'o')),
			Subsignal('dqs', DiffPairs('N2 U2', 'N1 V2', dir = 'io'), Attrs(IOSTANDARD = 'DIFF_SSTL135')),
			Subsignal(
				'dq',
				Pins('K5 L3 K3 L6 M3 M1 L4 M2 V4 T5 U4 V5 V1 T3 U3 R3', dir = 'io'),
				Attrs(IN_TERM = 'UNTUNED_SPLIT_40')
			),
			Subsignal('dm', Pins('L1 U1', dir = 'o')),
			Subsignal('odt', Pins('R5', dir = 'o')),
			Attrs(IOSTANDARD = 'SSTL135', SLEW = 'FAST'),
		),
		Resource(
			'eth_clk25', 0,
			Pins('G18', dir = 'o'), Clock(25e6), Attrs(IOSTANDARD = 'LVCMOS33')
		),
		Resource(
			'eth_clk50', 0, Pins('G18', dir = 'o'), Clock(50e6), Attrs(IOSTANDARD = 'LVCMOS33')
		),
		# TODO(aki): Replace with `EthernetResource` when more granular attrs are possible
		Resource(
			'eth_mii', 0,
			Subsignal('rst', PinsN('C16', dir = 'o')),
			Subsignal('mdio', Pins('K13', dir = 'io')),
			Subsignal('mdc', Pins('F16', dir = 'o')),
			Subsignal('tx_clk', Pins('H16', dir = 'i')),
			Subsignal('tx_en', Pins('H15', dir = 'o')),
			Subsignal('tx_data', Pins('H14 J14 J13 H17', dir = 'o')),
			Subsignal('rx_clk', Pins('F15', dir = 'i')),
			Subsignal('rx_dv', Pins('G16', dir = 'i'), Attrs(PULLDOWN = 'TRUE')), # strap to select MII
			Subsignal('rx_er', Pins('C17', dir = 'i')),
			Subsignal('rx_data', Pins('D18 E17 E18 G17', dir = 'i')),
			Subsignal('col', Pins('D17', dir = 'i')),
			Subsignal('crs', Pins('G14', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		),
		# TODO(aki): Replace with `EthernetResource` when more granular attrs are possible
		Resource(
			'eth_rmii', 0,
			Subsignal('rst', PinsN('C16', dir = 'o')),
			Subsignal('mdio', Pins('K13', dir = 'io')),
			Subsignal('mdc', Pins('F16', dir = 'o')),
			Subsignal('tx_en', Pins('H15', dir = 'o')),
			Subsignal('tx_data', Pins('H14 J14', dir = 'o')),
			Subsignal('rx_crs_dv', Pins('G14', dir = 'i')),
			Subsignal('rx_dv', Pins('G16', dir = 'i'), Attrs(PULLUP = 'TRUE')), # strap to select RMII
			Subsignal('rx_er', Pins('C17', dir = 'i')),
			Subsignal('rx_data', Pins('D18 E17', dir = 'i')),
			Attrs(IOSTANDARD = 'LVCMOS33')
		)
	]
	connectors  = [
		Connector('pmod', 0, 'G13 B11 A11 D12 - - D13 B18 A18 K16 - -'), # JA
		Connector('pmod', 1, 'E15 E16 D15 C15 - - J17 J18 K15 J15 - -'), # JB
		Connector('pmod', 2, 'U12 V12 V10 V11 - - U14 V14 T13 U13 - -'), # JC
		Connector('pmod', 3, 'D4 D3 F4 F3 - -  E2 D2 H2 G2 - -'), # JD
		Connector('ck_io', 0, {
			# Outer Digital Header
			'io0': 'V15',
			'io1': 'U16',
			'io2': 'P14',
			'io3': 'T11',
			'io4': 'R12',
			'io5': 'T14',
			'io6': 'T15',
			'io7': 'T16',
			'io8': 'N15',
			'io9': 'M16',
			'io10': 'V17',
			'io11': 'U18',
			'io12': 'R17',
			'io13': 'P17',
			# Inner Digital Header
			'io26': 'U11',
			'io27': 'V16',
			'io28': 'M13',
			'io29': 'R10',
			'io30': 'R11',
			'io31': 'R13',
			'io32': 'R15',
			'io33': 'P15',
			'io34': 'R16',
			'io35': 'N16',
			'io36': 'N14',
			'io37': 'U17',
			'io38': 'T18',
			'io39': 'R18',
			'io40': 'P18',
			'io41': 'N17',
			# Outer Analog Header as Digital IO
			'a0': 'F5',
			'a1': 'D8',
			'a2': 'C7',
			'a3': 'E7',
			'a4': 'D7',
			'a5': 'D5',
			# Inner Analog Header as Digital IO
			'io20': 'B7',
			'io21': 'B6',
			'io22': 'E6',
			'io23': 'E5',
			'io24': 'A4',
			'io25': 'A3'
		}),
		Connector('xadc', 0, {
			# Outer Analog Header
			'vaux4_n': 'C5',
			'vaux4_p': 'C6',
			'vaux5_n': 'A5',
			'vaux5_p': 'A6',
			'vaux6_n': 'B4',
			'vaux6_p': 'C4',
			'vaux7_n': 'A1',
			'vaux7_p': 'B1',
			'vaux15_n': 'B2',
			'vaux15_p': 'B3',
			'vaux0_n': 'C14',
			'vaux0_p': 'D14',
			# Inner Analog Header
			'vaux12_n': 'B7',
			'vaux12_p': 'B6',
			'vaux13_n': 'E6',
			'vaux13_p': 'E5',
			'vaux14_n': 'A4',
			'vaux14_p': 'A3',
			# Power Measurements
			'vsnsuv_n': 'B17',
			'vsnsuv_p': 'B16',
			'vsns5v0_n': 'B12',
			'vsns5v0_p': 'C12',
			'isns5v0_n': 'F14',
			'isns5v0_p': 'F13',
			'isns0v95_n': 'A16',
			'isns0v95_p': 'A15',
		})
	]

	def toolchain_prepare(self, fragment: Fragment, name: str, **kwargs) -> BuildPlan:
		overrides = {
			'script_before_bitstream': 'set_property BITSTREAM.CONFIG.SPI_BUSWIDTH 4 [current_design]',
			'script_after_bitstream': dedent(f'''
				write_cfgmem -force -format bin -interface spix4 -size 16 -loadbit 'up 0x0 {name}.bit' -file {name}.bin
			'''),
			'add_constraints': dedent('''
				set_property INTERNAL_VREF 0.675 [get_iobanks 34]
				set_property CFGBVS VCCO [current_design]
				set_property CONFIG_VOLTAGE 3.3 [current_design]
			''')
		}
		return super().toolchain_prepare(fragment, name, **overrides, **kwargs)

	def toolchain_program(self, products: BuildProducts, name: str) -> None:
		from os         import environ
		from subprocess import run

		xc3sprog = environ.get('XC3SPROG', 'xc3sprog')
		with products.extract(f'{name}.bit') as bitstream_filename:
			run([xc3sprog, '-c', 'nexys4', bitstream_filename], check = True)


class ArtyA7_35Platform(_ArtyA7Platform):
	device = 'xc7a35ti'

	pretty_name = 'Arty A7-35T'
	description = 'Digilent Arty A7-35T Xilinx Artix7-35 Development Board'

class ArtyA7_100Platform(_ArtyA7Platform):
	device = 'xc7a100ti'

	pretty_name = 'Arty A7-100T'
	description = 'Digilent Arty-A7-100T Xilinx Artix7-100 Development Board'

if __name__ == '__main__':
	from ...test.blinky import Blinky
	ArtyA7_35Platform().build(Blinky(), do_program = True)



# SPDX-License-Identifier: BSD-2-Clause

from .arty_z7          import ArtyZ720Platform
from .ebaz4205         import EBAZ4205Platform
from .microzed_z010    import MicroZedZ010Platform
from .microzed_z020    import MicroZedZ020Platform
from .zturn_lite_z007s import ZTurnLiteZ007SPlatform
from .zturn_lite_z010  import ZTurnLiteZ010Platform

__all__ = (
	'ArtyZ720Platform',
	'EBAZ4205Platform',
	'MicroZedZ010Platform',
	'MicroZedZ020Platform',
	'ZTurnLiteZ007SPlatform',
	'ZTurnLiteZ010Platform',
)
